# Copyright 2021 Vincent Texier <vit@free.fr>
#
# This software is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
import sys
from collections import OrderedDict
from pathlib import Path

# Path constants
DATA_PATH = Path("~/.config/tikka")

# Standalone executable install
if getattr(sys, "frozen", False):
    PACKAGE_PATH = Path(sys.executable).parent
# Python package install
else:
    PACKAGE_PATH = Path(__file__).parents[2]

LOCALES_PATH = PACKAGE_PATH.joinpath("locales")
CONFIG_FILENAME = "config.json"
CURRENCIES_FILENAME = "currencies.yaml"
DATABASE_FILE_EXTENSION = ".sqlite3"
DATABASE_MIGRATIONS_PATH = PACKAGE_PATH.joinpath(
    "adapters/repository/assets/migrations"
)

# Constants
LANGUAGES = OrderedDict([("en_US", "English"), ("fr_FR", "Français")])
MNEMONIC_LANGUAGES = {"en_US": "english", "fr_FR": "french"}
MNEMONIC_WORDS_LENGTH = 12
ACCESS_TYPE_MNEMONIC = "mnemonic"
ACCESS_TYPE_CLASSIC = "classic"
WALLETS_PASSWORD_LENGTH = 6
WALLETS_NONCE_SIZE = 12
PASSWORDS_NONCE_SIZE = 12
AMOUNT_UNIT_KEY = "unit"

# default derivation path
DERIVATION_PATH_TRANSPARENT_DEFAULT = "//2"
DERIVATION_PATH_OPAQUE_DEFAULT = "//1"
DERIVATION_PATH_MEMBER = "//0"
DERIVATION_SCAN_MAX_NUMBER = 50

# preferences keys
NODES_CURRENT_ENTRY_POINT_URL_PREFERENCES_KEY = "nodes_current_entry_point_url"

# servers
SMITH_NODE_URL = "ws://localhost:9944"



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'tikka/slots/pyqt/resources/gui/widgets/currency.ui'
#
# Created by: PyQt5 UI code generator 5.15.9
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_currencyWidget(object):
    def setupUi(self, currencyWidget):
        currencyWidget.setObjectName("currencyWidget")
        currencyWidget.resize(724, 475)
        self.verticalLayout = QtWidgets.QVBoxLayout(currencyWidget)
        self.verticalLayout.setObjectName("verticalLayout")
        self.refreshButton = QtWidgets.QPushButton(currencyWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.refreshButton.sizePolicy().hasHeightForWidth())
        self.refreshButton.setSizePolicy(sizePolicy)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/icons/refresh_icon"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.refreshButton.setIcon(icon)
        self.refreshButton.setObjectName("refreshButton")
        self.verticalLayout.addWidget(self.refreshButton, 0, QtCore.Qt.AlignRight)
        self.groupBox = QtWidgets.QGroupBox(currencyWidget)
        self.groupBox.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.groupBox.setObjectName("groupBox")
        self.formLayout_2 = QtWidgets.QFormLayout(self.groupBox)
        self.formLayout_2.setLabelAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.formLayout_2.setObjectName("formLayout_2")
        self.nameLabel = QtWidgets.QLabel(self.groupBox)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.nameLabel.setFont(font)
        self.nameLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.nameLabel.setObjectName("nameLabel")
        self.formLayout_2.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.nameLabel)
        self.universalDividendLabel = QtWidgets.QLabel(self.groupBox)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.universalDividendLabel.setFont(font)
        self.universalDividendLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.universalDividendLabel.setObjectName("universalDividendLabel")
        self.formLayout_2.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.universalDividendLabel)
        self.symbolLabel = QtWidgets.QLabel(self.groupBox)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.symbolLabel.setFont(font)
        self.symbolLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.symbolLabel.setObjectName("symbolLabel")
        self.formLayout_2.setWidget(2, QtWidgets.QFormLayout.LabelRole, self.symbolLabel)
        self.monetaryMassLabel = QtWidgets.QLabel(self.groupBox)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.monetaryMassLabel.setFont(font)
        self.monetaryMassLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.monetaryMassLabel.setObjectName("monetaryMassLabel")
        self.formLayout_2.setWidget(3, QtWidgets.QFormLayout.LabelRole, self.monetaryMassLabel)
        self.membersLabel = QtWidgets.QLabel(self.groupBox)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.membersLabel.setFont(font)
        self.membersLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.membersLabel.setObjectName("membersLabel")
        self.formLayout_2.setWidget(4, QtWidgets.QFormLayout.LabelRole, self.membersLabel)
        self.membersValueLabel = QtWidgets.QLabel(self.groupBox)
        self.membersValueLabel.setText("")
        self.membersValueLabel.setObjectName("membersValueLabel")
        self.formLayout_2.setWidget(4, QtWidgets.QFormLayout.FieldRole, self.membersValueLabel)
        self.universalDividendValueLabel = QtWidgets.QLabel(self.groupBox)
        self.universalDividendValueLabel.setText("")
        self.universalDividendValueLabel.setObjectName("universalDividendValueLabel")
        self.formLayout_2.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.universalDividendValueLabel)
        self.monetaryMassValueLabel = QtWidgets.QLabel(self.groupBox)
        self.monetaryMassValueLabel.setText("")
        self.monetaryMassValueLabel.setObjectName("monetaryMassValueLabel")
        self.formLayout_2.setWidget(3, QtWidgets.QFormLayout.FieldRole, self.monetaryMassValueLabel)
        self.symbolValueLabel = QtWidgets.QLabel(self.groupBox)
        self.symbolValueLabel.setText("")
        self.symbolValueLabel.setObjectName("symbolValueLabel")
        self.formLayout_2.setWidget(2, QtWidgets.QFormLayout.FieldRole, self.symbolValueLabel)
        self.nameValueLabel = QtWidgets.QLabel(self.groupBox)
        self.nameValueLabel.setText("")
        self.nameValueLabel.setObjectName("nameValueLabel")
        self.formLayout_2.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.nameValueLabel)
        self.verticalLayout.addWidget(self.groupBox)
        self.errorLabel = QtWidgets.QLabel(currencyWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.errorLabel.sizePolicy().hasHeightForWidth())
        self.errorLabel.setSizePolicy(sizePolicy)
        self.errorLabel.setStyleSheet("color: red")
        self.errorLabel.setText("")
        self.errorLabel.setAlignment(QtCore.Qt.AlignCenter)
        self.errorLabel.setObjectName("errorLabel")
        self.verticalLayout.addWidget(self.errorLabel)
        self.groupBox_2 = QtWidgets.QGroupBox(currencyWidget)
        self.groupBox_2.setObjectName("groupBox_2")
        self.formLayout_3 = QtWidgets.QFormLayout(self.groupBox_2)
        self.formLayout_3.setObjectName("formLayout_3")
        self.expectedBlockDurationLabel = QtWidgets.QLabel(self.groupBox_2)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.expectedBlockDurationLabel.setFont(font)
        self.expectedBlockDurationLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.expectedBlockDurationLabel.setObjectName("expectedBlockDurationLabel")
        self.formLayout_3.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.expectedBlockDurationLabel)
        self.expectedBlockDurationValueLabel = QtWidgets.QLabel(self.groupBox_2)
        self.expectedBlockDurationValueLabel.setText("")
        self.expectedBlockDurationValueLabel.setObjectName("expectedBlockDurationValueLabel")
        self.formLayout_3.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.expectedBlockDurationValueLabel)
        self.expectedEpochDurationLabel = QtWidgets.QLabel(self.groupBox_2)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.expectedEpochDurationLabel.setFont(font)
        self.expectedEpochDurationLabel.setObjectName("expectedEpochDurationLabel")
        self.formLayout_3.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.expectedEpochDurationLabel)
        self.expectedEpochDurationValueLabel = QtWidgets.QLabel(self.groupBox_2)
        self.expectedEpochDurationValueLabel.setText("")
        self.expectedEpochDurationValueLabel.setObjectName("expectedEpochDurationValueLabel")
        self.formLayout_3.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.expectedEpochDurationValueLabel)
        self.verticalLayout.addWidget(self.groupBox_2)

        self.retranslateUi(currencyWidget)
        QtCore.QMetaObject.connectSlotsByName(currencyWidget)

    def retranslateUi(self, currencyWidget):
        #  _translate = QtCore.QCoreApplication.translate
        currencyWidget.setWindowTitle(_("Currency parameters"))
        self.refreshButton.setText(_("Refresh"))
        self.groupBox.setTitle(_("Currency"))
        self.nameLabel.setText(_("Name"))
        self.universalDividendLabel.setText(_("Universal Dividend"))
        self.symbolLabel.setText(_("Symbol"))
        self.monetaryMassLabel.setText(_("Monetary Mass"))
        self.membersLabel.setText(_("Members"))
        self.groupBox_2.setTitle(_("Blockchain"))
        self.expectedBlockDurationLabel.setText(_("Expected Block duration"))
        self.expectedEpochDurationLabel.setText(_("Expected Epoch duration"))
import tikka.slots.pyqt.resources.icons.index_rc



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'tikka/slots/pyqt/resources/gui/windows/welcome.ui'
#
# Created by: PyQt5 UI code generator 5.15.9
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_WelcomeDialog(object):
    def setupUi(self, WelcomeDialog):
        WelcomeDialog.setObjectName("WelcomeDialog")
        WelcomeDialog.resize(505, 672)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(WelcomeDialog.sizePolicy().hasHeightForWidth())
        WelcomeDialog.setSizePolicy(sizePolicy)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(WelcomeDialog)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.label = QtWidgets.QLabel(WelcomeDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label.sizePolicy().hasHeightForWidth())
        self.label.setSizePolicy(sizePolicy)
        self.label.setPixmap(QtGui.QPixmap(":/icons/logo"))
        self.label.setAlignment(QtCore.Qt.AlignCenter)
        self.label.setWordWrap(False)
        self.label.setObjectName("label")
        self.verticalLayout_2.addWidget(self.label)
        self.label_3 = QtWidgets.QLabel(WelcomeDialog)
        self.label_3.setAlignment(QtCore.Qt.AlignCenter)
        self.label_3.setObjectName("label_3")
        self.verticalLayout_2.addWidget(self.label_3)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setContentsMargins(-1, 20, -1, 20)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.importButton = QtWidgets.QPushButton(WelcomeDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.importButton.sizePolicy().hasHeightForWidth())
        self.importButton.setSizePolicy(sizePolicy)
        self.importButton.setObjectName("importButton")
        self.horizontalLayout.addWidget(self.importButton)
        self.createButton = QtWidgets.QPushButton(WelcomeDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.createButton.sizePolicy().hasHeightForWidth())
        self.createButton.setSizePolicy(sizePolicy)
        self.createButton.setObjectName("createButton")
        self.horizontalLayout.addWidget(self.createButton)
        self.verticalLayout_2.addLayout(self.horizontalLayout)
        self.label_2 = QtWidgets.QLabel(WelcomeDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_2.sizePolicy().hasHeightForWidth())
        self.label_2.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setItalic(True)
        self.label_2.setFont(font)
        self.label_2.setAutoFillBackground(False)
        self.label_2.setScaledContents(True)
        self.label_2.setWordWrap(True)
        self.label_2.setObjectName("label_2")
        self.verticalLayout_2.addWidget(self.label_2)
        self.continueButton = QtWidgets.QPushButton(WelcomeDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.continueButton.sizePolicy().hasHeightForWidth())
        self.continueButton.setSizePolicy(sizePolicy)
        self.continueButton.setObjectName("continueButton")
        self.verticalLayout_2.addWidget(self.continueButton, 0, QtCore.Qt.AlignHCenter)

        self.retranslateUi(WelcomeDialog)
        QtCore.QMetaObject.connectSlotsByName(WelcomeDialog)
        WelcomeDialog.setTabOrder(self.importButton, self.createButton)
        WelcomeDialog.setTabOrder(self.createButton, self.continueButton)

    def retranslateUi(self, WelcomeDialog):
        #  _translate = QtCore.QCoreApplication.translate
        WelcomeDialog.setWindowTitle(_("Welcome"))
        self.label_3.setText(_("Welcome in Tikka, the Duniter V2 client for the desktop.\n"
"\n"
"To get started, you can import an existing account or create a new one.*"))
        self.importButton.setText(_("Import existing account"))
        self.createButton.setText(_("Create a new account"))
        self.label_2.setText(_("* If you are a V1 user, without a V2 account, you need to create a new V2 account before importing your V1 account into it."))
        self.continueButton.setText(_("Continue"))
import tikka.slots.pyqt.resources.icons.index_rc



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'tikka/slots/pyqt/resources/gui/widgets/technical_committee_proposal.ui'
#
# Created by: PyQt5 UI code generator 5.15.9
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_TechnicalCommitteeProposalWidget(object):
    def setupUi(self, TechnicalCommitteeProposalWidget):
        TechnicalCommitteeProposalWidget.setObjectName("TechnicalCommitteeProposalWidget")
        TechnicalCommitteeProposalWidget.resize(675, 227)
        self.gridLayout_2 = QtWidgets.QGridLayout(TechnicalCommitteeProposalWidget)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.proposalGroupBox = QtWidgets.QGroupBox(TechnicalCommitteeProposalWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.proposalGroupBox.sizePolicy().hasHeightForWidth())
        self.proposalGroupBox.setSizePolicy(sizePolicy)
        self.proposalGroupBox.setObjectName("proposalGroupBox")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.proposalGroupBox)
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.callGroupBox = QtWidgets.QGroupBox(self.proposalGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.callGroupBox.sizePolicy().hasHeightForWidth())
        self.callGroupBox.setSizePolicy(sizePolicy)
        self.callGroupBox.setObjectName("callGroupBox")
        self.formLayout_2 = QtWidgets.QFormLayout(self.callGroupBox)
        self.formLayout_2.setLabelAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.formLayout_2.setObjectName("formLayout_2")
        self.callIndexLabel = QtWidgets.QLabel(self.callGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.callIndexLabel.sizePolicy().hasHeightForWidth())
        self.callIndexLabel.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.callIndexLabel.setFont(font)
        self.callIndexLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.callIndexLabel.setObjectName("callIndexLabel")
        self.formLayout_2.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.callIndexLabel)
        self.callIndexValueLabel = QtWidgets.QLabel(self.callGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.callIndexValueLabel.sizePolicy().hasHeightForWidth())
        self.callIndexValueLabel.setSizePolicy(sizePolicy)
        self.callIndexValueLabel.setObjectName("callIndexValueLabel")
        self.formLayout_2.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.callIndexValueLabel)
        self.callHashLabel = QtWidgets.QLabel(self.callGroupBox)
        self.callHashLabel.setEnabled(True)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.callHashLabel.sizePolicy().hasHeightForWidth())
        self.callHashLabel.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.callHashLabel.setFont(font)
        self.callHashLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.callHashLabel.setObjectName("callHashLabel")
        self.formLayout_2.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.callHashLabel)
        self.callHashValueLabel = QtWidgets.QLabel(self.callGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.callHashValueLabel.sizePolicy().hasHeightForWidth())
        self.callHashValueLabel.setSizePolicy(sizePolicy)
        self.callHashValueLabel.setObjectName("callHashValueLabel")
        self.formLayout_2.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.callHashValueLabel)
        self.callModuleLabel = QtWidgets.QLabel(self.callGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.callModuleLabel.sizePolicy().hasHeightForWidth())
        self.callModuleLabel.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.callModuleLabel.setFont(font)
        self.callModuleLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.callModuleLabel.setObjectName("callModuleLabel")
        self.formLayout_2.setWidget(2, QtWidgets.QFormLayout.LabelRole, self.callModuleLabel)
        self.callModuleValueLabel = QtWidgets.QLabel(self.callGroupBox)
        self.callModuleValueLabel.setObjectName("callModuleValueLabel")
        self.formLayout_2.setWidget(2, QtWidgets.QFormLayout.FieldRole, self.callModuleValueLabel)
        self.callFunctionLabel = QtWidgets.QLabel(self.callGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.callFunctionLabel.sizePolicy().hasHeightForWidth())
        self.callFunctionLabel.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.callFunctionLabel.setFont(font)
        self.callFunctionLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.callFunctionLabel.setObjectName("callFunctionLabel")
        self.formLayout_2.setWidget(3, QtWidgets.QFormLayout.LabelRole, self.callFunctionLabel)
        self.callFunctionValueLabel = QtWidgets.QLabel(self.callGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.callFunctionValueLabel.sizePolicy().hasHeightForWidth())
        self.callFunctionValueLabel.setSizePolicy(sizePolicy)
        self.callFunctionValueLabel.setObjectName("callFunctionValueLabel")
        self.formLayout_2.setWidget(3, QtWidgets.QFormLayout.FieldRole, self.callFunctionValueLabel)
        self.callArgumentsLabel = QtWidgets.QLabel(self.callGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.callArgumentsLabel.sizePolicy().hasHeightForWidth())
        self.callArgumentsLabel.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.callArgumentsLabel.setFont(font)
        self.callArgumentsLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.callArgumentsLabel.setObjectName("callArgumentsLabel")
        self.formLayout_2.setWidget(4, QtWidgets.QFormLayout.LabelRole, self.callArgumentsLabel)
        self.callArgumentsValueLabel = QtWidgets.QLabel(self.callGroupBox)
        self.callArgumentsValueLabel.setObjectName("callArgumentsValueLabel")
        self.formLayout_2.setWidget(4, QtWidgets.QFormLayout.FieldRole, self.callArgumentsValueLabel)
        self.horizontalLayout_4.addWidget(self.callGroupBox)
        self.votingGroupBox = QtWidgets.QGroupBox(self.proposalGroupBox)
        self.votingGroupBox.setObjectName("votingGroupBox")
        self.formLayout = QtWidgets.QFormLayout(self.votingGroupBox)
        self.formLayout.setLabelAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.formLayout.setObjectName("formLayout")
        self.votingIndexLabel = QtWidgets.QLabel(self.votingGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.votingIndexLabel.sizePolicy().hasHeightForWidth())
        self.votingIndexLabel.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.votingIndexLabel.setFont(font)
        self.votingIndexLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.votingIndexLabel.setObjectName("votingIndexLabel")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.votingIndexLabel)
        self.votingIndexValueLabel = QtWidgets.QLabel(self.votingGroupBox)
        self.votingIndexValueLabel.setObjectName("votingIndexValueLabel")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.votingIndexValueLabel)
        self.votingThresholdLabel = QtWidgets.QLabel(self.votingGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.votingThresholdLabel.sizePolicy().hasHeightForWidth())
        self.votingThresholdLabel.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.votingThresholdLabel.setFont(font)
        self.votingThresholdLabel.setObjectName("votingThresholdLabel")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.votingThresholdLabel)
        self.votingThresholdValueLabel = QtWidgets.QLabel(self.votingGroupBox)
        self.votingThresholdValueLabel.setObjectName("votingThresholdValueLabel")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.votingThresholdValueLabel)
        self.votingAyesValueLabel = QtWidgets.QLabel(self.votingGroupBox)
        self.votingAyesValueLabel.setObjectName("votingAyesValueLabel")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.FieldRole, self.votingAyesValueLabel)
        self.votingNaysValueLabel = QtWidgets.QLabel(self.votingGroupBox)
        self.votingNaysValueLabel.setObjectName("votingNaysValueLabel")
        self.formLayout.setWidget(3, QtWidgets.QFormLayout.FieldRole, self.votingNaysValueLabel)
        self.votingEndLabel = QtWidgets.QLabel(self.votingGroupBox)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.votingEndLabel.setFont(font)
        self.votingEndLabel.setObjectName("votingEndLabel")
        self.formLayout.setWidget(5, QtWidgets.QFormLayout.LabelRole, self.votingEndLabel)
        self.votingEndValueLabel = QtWidgets.QLabel(self.votingGroupBox)
        self.votingEndValueLabel.setObjectName("votingEndValueLabel")
        self.formLayout.setWidget(5, QtWidgets.QFormLayout.FieldRole, self.votingEndValueLabel)
        self.yesButton = QtWidgets.QPushButton(self.votingGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.yesButton.sizePolicy().hasHeightForWidth())
        self.yesButton.setSizePolicy(sizePolicy)
        self.yesButton.setObjectName("yesButton")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.LabelRole, self.yesButton)
        self.noButton = QtWidgets.QPushButton(self.votingGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.noButton.sizePolicy().hasHeightForWidth())
        self.noButton.setSizePolicy(sizePolicy)
        self.noButton.setObjectName("noButton")
        self.formLayout.setWidget(3, QtWidgets.QFormLayout.LabelRole, self.noButton)
        self.horizontalLayout_4.addWidget(self.votingGroupBox)
        self.verticalLayout.addLayout(self.horizontalLayout_4)
        self.gridLayout_2.addWidget(self.proposalGroupBox, 0, 0, 1, 1)

        self.retranslateUi(TechnicalCommitteeProposalWidget)
        QtCore.QMetaObject.connectSlotsByName(TechnicalCommitteeProposalWidget)

    def retranslateUi(self, TechnicalCommitteeProposalWidget):
        #  _translate = QtCore.QCoreApplication.translate
        TechnicalCommitteeProposalWidget.setWindowTitle(_("Form"))
        self.proposalGroupBox.setTitle(_("Proposal Hash"))
        self.callGroupBox.setTitle(_("Call"))
        self.callIndexLabel.setText(_("Index"))
        self.callIndexValueLabel.setText(_("0"))
        self.callHashLabel.setText(_("Hash"))
        self.callHashValueLabel.setText(_("xxx"))
        self.callModuleLabel.setText(_("Module"))
        self.callModuleValueLabel.setText(_("TextLabel"))
        self.callFunctionLabel.setText(_("Function"))
        self.callFunctionValueLabel.setText(_("TextLabel"))
        self.callArgumentsLabel.setText(_("Arguments"))
        self.callArgumentsValueLabel.setText(_("TextLabel"))
        self.votingGroupBox.setTitle(_("Voting"))
        self.votingIndexLabel.setText(_("index"))
        self.votingIndexValueLabel.setText(_("0"))
        self.votingThresholdLabel.setText(_("Threshold"))
        self.votingThresholdValueLabel.setText(_("0"))
        self.votingAyesValueLabel.setText(_("TextLabel"))
        self.votingNaysValueLabel.setText(_("TextLabel"))
        self.votingEndLabel.setText(_("End"))
        self.votingEndValueLabel.setText(_("TextLabel"))
        self.yesButton.setText(_("Yes"))
        self.noButton.setText(_("No"))



# Copyright 2021 Vincent Texier <vit@free.fr>
#
# This software is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
import sys
from typing import Optional

from PyQt5.QtCore import QModelIndex, QMutex, QPoint, QRect
from PyQt5.QtGui import QPainter, QPixmap
from PyQt5.QtWidgets import (
    QApplication,
    QMainWindow,
    QStyledItemDelegate,
    QStyleOptionViewItem,
    QWidget,
)

from tikka.domains.application import Application
from tikka.domains.entities.account import Account
from tikka.domains.entities.category import Category
from tikka.domains.entities.constants import AMOUNT_UNIT_KEY, DATA_PATH
from tikka.domains.entities.events import (
    AccountEvent,
    CategoryEvent,
    CurrencyEvent,
    UnitEvent,
)
from tikka.slots.pyqt.entities.constants import SELECTED_UNIT_PREFERENCES_KEY
from tikka.slots.pyqt.entities.worker import AsyncQWorker
from tikka.slots.pyqt.models.account_tree import AccountTreeModel, MemberStatusRole
from tikka.slots.pyqt.resources.gui.widgets.account_tree_rc import Ui_AccountTreeWidget
from tikka.slots.pyqt.widgets.account_menu import AccountPopupMenu
from tikka.slots.pyqt.widgets.category_menu import CategoryPopupMenu


class AccountIconsItemDelegate(QStyledItemDelegate):
    """
    Class used to display member status icon for accounts
    """

    def paint(
        self, painter: QPainter, option: QStyleOptionViewItem, index: QModelIndex
    ) -> None:
        """
        Draw member status icon and wallet lock status icon side by side

        :param painter: QPainter instance
        :param option: QStyleOptionViewItem instance
        :param index: QModelIndex instance
        :return:
        """
        super().paint(painter, option, index)
        member_pixmap: QPixmap = index.data(MemberStatusRole)
        if isinstance(member_pixmap, QPixmap):
            member_rect = QRect(
                option.rect.x() - member_pixmap.rect().width() - 2,
                option.rect.y(),
                member_pixmap.rect().width(),
                member_pixmap.rect().height(),
            )
            painter.drawPixmap(member_rect, member_pixmap)


class AccountTreeWidget(QWidget, Ui_AccountTreeWidget):
    """
    AccountTreeWidget class
    """

    def __init__(
        self,
        application: Application,
        mutex: QMutex,
        parent: Optional[QWidget] = None,
    ) -> None:
        """
        Init AccountTreeWidget instance

        :param application: Application instance
        :param mutex: QMutex instance
        :param parent: QWidget instance
        """
        super().__init__(parent=parent)
        self.setupUi(self)

        self.application = application
        self._ = self.application.translator.gettext
        self.mutex = mutex
        self.account_tree_model = AccountTreeModel(self.application, self.locale())
        self.treeView.setModel(self.account_tree_model)
        self.treeView.setHeaderHidden(True)
        self.treeView.expandAll()
        self.treeView.resizeColumnToContents(0)
        # set style sheet
        self.treeView.setStyleSheet("QTreeView::item {" "padding: 5px;" "}")
        self.treeView.setItemDelegateForColumn(0, AccountIconsItemDelegate())
        # init expand/collapse state of categories
        self.recursive_expand_or_collapse_categories(QModelIndex())
        self.treeView.resizeColumnToContents(1)

        ##############################
        # ASYNC METHODS
        ##############################
        # Create a QWorker object
        self.fetch_from_network_async_qworker = AsyncQWorker(
            self.fetch_from_network, self.mutex
        )
        self.fetch_from_network_async_qworker.finished.connect(
            self._on_finished_fetch_from_network
        )

        # events
        self.addCategoryButton.clicked.connect(self.on_add_category_button_clicked)
        self.refreshButton.clicked.connect(self.fetch_from_network_async_qworker.start)
        self.treeView.customContextMenuRequested.connect(self.on_context_menu)
        self.treeView.expanded.connect(self.on_expanded)
        self.treeView.collapsed.connect(self.on_collapsed)
        self.treeView.model().dataChanged.connect(self.on_data_changed)

        # application events
        self.application.event_dispatcher.add_event_listener(
            CategoryEvent.EVENT_TYPE_ADD, self.on_category_add_event
        )
        self.application.event_dispatcher.add_event_listener(
            CategoryEvent.EVENT_TYPE_DELETE, self.on_category_delete_event
        )
        self.application.event_dispatcher.add_event_listener(
            UnitEvent.EVENT_TYPE_CHANGED, self.on_unit_event
        )
        self.application.event_dispatcher.add_event_listener(
            CurrencyEvent.EVENT_TYPE_CHANGED, self.on_currency_event
        )
        self.application.event_dispatcher.add_event_listener(
            AccountEvent.EVENT_TYPE_ADD, self.on_add_account_event
        )
        self.application.event_dispatcher.add_event_listener(
            AccountEvent.EVENT_TYPE_DELETE, self.on_delete_account_event
        )
        self.application.event_dispatcher.add_event_listener(
            AccountEvent.EVENT_TYPE_UPDATE, self.on_update_account_event
        )

    def rename_category(self):
        """
        Set the current selected category in edit mode in tree view

        :return:
        """
        index = self.treeView.currentIndex()
        item = index.internalPointer()
        if isinstance(item.element, Category):
            self.treeView.edit(self.treeView.currentIndex())

    def add_category(self, parent: QModelIndex):
        """
        Add a new category in index in edit mode

        :param parent: Parent QModelIndex instance
        :return:
        """
        self.treeView.model().insertRow(0, parent)
        self.recursive_expand_or_collapse_categories(QModelIndex())
        self.treeView.edit(self.treeView.model().index(0, 0, parent))

    def add_sub_category(self):
        """
        Add new sub category in current selected category

        :return:
        """
        index = self.treeView.currentIndex()
        item = index.internalPointer()
        if isinstance(item.element, Category):
            self.add_category(index)

    def on_data_changed(self, _):
        """
        Triggered when data changed in tree view model

        :return:
        """
        self._update_model()

    def recursive_expand_or_collapse_categories(self, index: QModelIndex) -> None:
        """
        Expand or collapse categories in tree view depending on expanded attribute

        :param index: Index to expand/collapse
        :return:
        """
        item = index.internalPointer()
        if item is not None:
            element = index.internalPointer().element
            if isinstance(element, Category):
                if element.expanded is True:
                    self.treeView.expand(index)
                else:
                    self.treeView.collapse(index)
            else:
                self.treeView.expand(index)
        for row in range(0, self.account_tree_model.rowCount(index)):
            self.recursive_expand_or_collapse_categories(
                self.account_tree_model.index(row, 0, index)
            )
        self.treeView.resizeColumnToContents(0)

    def on_expanded(self, index: QModelIndex) -> None:
        """
        Triggered when user expand a row

        :param index: QModelIndex instance of expanded row
        :return:
        """
        # get selected element
        element = index.internalPointer().element
        if isinstance(element, Category) and element.expanded is False:
            self.application.categories.expand(element)

    def on_collapsed(self, index: QModelIndex) -> None:
        """
        Triggered when user collapse a row

        :param index: QModelIndex instance of collapsed row
        :return:
        """
        # get selected element
        element = index.internalPointer().element
        if isinstance(element, Category) and element.expanded is True:
            self.application.categories.collapse(element)

    def _update_model(self):
        """
        Triggered when adding a category

        :return:
        """
        self.treeView.model().init_data()
        self.treeView.resizeColumnToContents(0)
        self.recursive_expand_or_collapse_categories(QModelIndex())

    def update_ui(self) -> None:
        """
        Update UI data

        :return:
        """
        unit_preference = self.application.preferences_repository.get(
            SELECTED_UNIT_PREFERENCES_KEY
        )
        if unit_preference is not None:
            amount = self.application.amounts.get_amount(unit_preference)
        else:
            amount = self.application.amounts.get_amount(AMOUNT_UNIT_KEY)

        self.totalBalanceValueLabel.setText(
            self.locale().toCurrencyString(
                amount.value(self.application.accounts.get_total_balance()),
                amount.symbol(),
            )
        )

    def on_add_category_button_clicked(self):
        """
        Triggered when user click on addCategoryButton

        :return:
        """
        self.add_category(self.treeView.rootIndex())

    def on_category_add_event(self, _):
        """
        When add category event is triggered

        :return:
        """
        self._update_model()

    def on_category_delete_event(self, _):
        """
        When delete category event is triggered

        :return:
        """
        self._update_model()
        self.update_ui()

    def on_unit_event(self, _):
        """
        When a unit event is triggered

        :return:
        """
        self._update_model()
        self.update_ui()

    def on_currency_event(self, _):
        """
        When a currency event is triggered

        :return:
        """
        self._update_model()
        self.update_ui()

    def on_add_account_event(self, _):
        """
        Add account row when account is created

        :return:
        """
        self._update_model()
        self.update_ui()

    def on_delete_account_event(self, _):
        """
        Remove account row when account is deleted

        :return:
        """
        self._update_model()
        self.update_ui()

    def on_update_account_event(self, _):
        """
        Update account row when account is updated

        :return:
        """
        self._update_model()
        self.update_ui()

    def on_context_menu(self, position: QPoint):
        """
        When right button on table widget

        :param position: QPoint instance
        :return:
        """
        # get selected account
        current_element = self.treeView.currentIndex().internalPointer().element
        if isinstance(current_element, Account):
            # display account popup menu at click position
            AccountPopupMenu(self.application, current_element, self.mutex, self).exec_(
                self.treeView.mapToGlobal(position)
            )
        else:
            # display category popup menu at click position
            CategoryPopupMenu(
                self.application, current_element, self.mutex, self
            ).exec_(self.treeView.mapToGlobal(position))

    def fetch_from_network(self):
        """
        Fetch table model accounts data from the network

        :return:
        """
        self.refreshButton.setEnabled(False)
        self.errorLabel.setText("")

        try:
            self.application.accounts.network_update_balances(
                self.application.accounts.get_list()
            )
        except Exception as exception:
            self.errorLabel.setText(self._(str(exception)))

        try:
            identity_indice = [
                identity_index
                for identity_index in self.application.identities.network_get_identity_indice(
                    [
                        account.address
                        for account in self.application.accounts.get_list()
                    ]
                )
                if identity_index is not None
            ]
        except Exception as exception:
            self.errorLabel.setText(self._(str(exception)))
        else:
            try:
                self.application.identities.network_get_identities(identity_indice)
            except Exception as exception:
                self.errorLabel.setText(self._(str(exception)))
            else:
                try:
                    self.application.smiths.network_get_smiths(identity_indice)
                except Exception as exception:
                    self.errorLabel.setText(self._(str(exception)))

    def _on_finished_fetch_from_network(self):
        """
        Triggered when async request fetch_from_network is finished

        :return:
        """
        self.refreshButton.setEnabled(True)
        self._update_model()
        self.update_ui()


if __name__ == "__main__":
    qapp = QApplication(sys.argv)

    application_ = Application(DATA_PATH)

    main_window = QMainWindow()
    main_window.showMaximized()

    main_window.setCentralWidget(AccountTreeWidget(application_, QMutex(), main_window))

    sys.exit(qapp.exec_())



# Copyright 2021 Vincent Texier <vit@free.fr>
#
# This software is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from dataclasses import dataclass
from datetime import datetime
from typing import List, TypeVar

TechnicalCommitteeVotingType = TypeVar(
    "TechnicalCommitteeVotingType", bound="TechnicalCommitteeVoting"
)


@dataclass
class TechnicalCommitteeVoting:

    index: int
    threshold: int
    ayes: List[str]
    nays: List[str]
    end: datetime


TechnicalCommitteeCallType = TypeVar(
    "TechnicalCommitteeCallType", bound="TechnicalCommitteeCall"
)


@dataclass
class TechnicalCommitteeCall:
    index: int
    hash: str
    module: str
    function: str
    args: str


TechnicalCommitteeProposalType = TypeVar(
    "TechnicalCommitteeProposalType", bound="TechnicalCommitteeProposal"
)


@dataclass
class TechnicalCommitteeProposal:
    hash: str
    call: TechnicalCommitteeCall
    voting: TechnicalCommitteeVoting

    def __str__(self):
        """
        Return string representation

        :return:
        """
        return f"{self.hash}"



# Copyright 2021 Vincent Texier <vit@free.fr>
#
# This software is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys
from typing import Optional

from PyQt5.QtWidgets import QApplication, QDialog, QWidget

from tikka import __version__
from tikka.domains.application import Application
from tikka.domains.entities.constants import DATA_PATH
from tikka.slots.pyqt.resources.gui.windows.about_rc import Ui_AboutDialog

AUTHORS = [
    "Vincent Texier",
]


class AboutWindow(QDialog, Ui_AboutDialog):
    """
    AboutWindow class
    """

    def __init__(self, parent: Optional[QWidget] = None):
        """
        Init about window

        :param parent: QWidget instance
        """
        super().__init__(parent=parent)
        self.setupUi(self)

        self.versionLabel.setText(f"Version {__version__}")
        self.authorListLabel.setText("\n".join(AUTHORS))


if __name__ == "__main__":
    qapp = QApplication(sys.argv)
    application_ = Application(DATA_PATH)
    AboutWindow().exec_()



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'tikka/slots/pyqt/resources/gui/widgets/smith.ui'
#
# Created by: PyQt5 UI code generator 5.15.9
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_SmithWidget(object):
    def setupUi(self, SmithWidget):
        SmithWidget.setObjectName("SmithWidget")
        SmithWidget.resize(743, 684)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(SmithWidget.sizePolicy().hasHeightForWidth())
        SmithWidget.setSizePolicy(sizePolicy)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(SmithWidget)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.label = QtWidgets.QLabel(SmithWidget)
        font = QtGui.QFont()
        font.setItalic(True)
        self.label.setFont(font)
        self.label.setObjectName("label")
        self.verticalLayout_2.addWidget(self.label, 0, QtCore.Qt.AlignHCenter)
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setObjectName("gridLayout")
        self.goOnlineButton = QtWidgets.QPushButton(SmithWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.goOnlineButton.sizePolicy().hasHeightForWidth())
        self.goOnlineButton.setSizePolicy(sizePolicy)
        self.goOnlineButton.setObjectName("goOnlineButton")
        self.gridLayout.addWidget(self.goOnlineButton, 2, 2, 1, 1)
        self.authorityValueLabel = QtWidgets.QLabel(SmithWidget)
        self.authorityValueLabel.setText("")
        self.authorityValueLabel.setObjectName("authorityValueLabel")
        self.gridLayout.addWidget(self.authorityValueLabel, 2, 1, 1, 1)
        self.accountComboBox = QtWidgets.QComboBox(SmithWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.accountComboBox.sizePolicy().hasHeightForWidth())
        self.accountComboBox.setSizePolicy(sizePolicy)
        self.accountComboBox.setToolTip("")
        self.accountComboBox.setEditable(False)
        self.accountComboBox.setMinimumContentsLength(0)
        self.accountComboBox.setObjectName("accountComboBox")
        self.gridLayout.addWidget(self.accountComboBox, 0, 1, 1, 1)
        self.accountLabel = QtWidgets.QLabel(SmithWidget)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.accountLabel.setFont(font)
        self.accountLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.accountLabel.setObjectName("accountLabel")
        self.gridLayout.addWidget(self.accountLabel, 0, 0, 1, 1)
        self.membershipLabel = QtWidgets.QLabel(SmithWidget)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.membershipLabel.setFont(font)
        self.membershipLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.membershipLabel.setObjectName("membershipLabel")
        self.gridLayout.addWidget(self.membershipLabel, 1, 0, 1, 1)
        self.membershipValueLabel = QtWidgets.QLabel(SmithWidget)
        self.membershipValueLabel.setText("")
        self.membershipValueLabel.setObjectName("membershipValueLabel")
        self.gridLayout.addWidget(self.membershipValueLabel, 1, 1, 1, 1)
        self.refreshSmithButton = QtWidgets.QPushButton(SmithWidget)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/icons/refresh_icon"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.refreshSmithButton.setIcon(icon)
        self.refreshSmithButton.setObjectName("refreshSmithButton")
        self.gridLayout.addWidget(self.refreshSmithButton, 0, 2, 1, 1)
        self.authorityLabel = QtWidgets.QLabel(SmithWidget)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.authorityLabel.setFont(font)
        self.authorityLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.authorityLabel.setObjectName("authorityLabel")
        self.gridLayout.addWidget(self.authorityLabel, 2, 0, 1, 1)
        self.goOfflineButton = QtWidgets.QPushButton(SmithWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.goOfflineButton.sizePolicy().hasHeightForWidth())
        self.goOfflineButton.setSizePolicy(sizePolicy)
        self.goOfflineButton.setObjectName("goOfflineButton")
        self.gridLayout.addWidget(self.goOfflineButton, 3, 2, 1, 1)
        self.loaderIconLabel = QtWidgets.QLabel(SmithWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.loaderIconLabel.sizePolicy().hasHeightForWidth())
        self.loaderIconLabel.setSizePolicy(sizePolicy)
        self.loaderIconLabel.setMaximumSize(QtCore.QSize(16, 16))
        self.loaderIconLabel.setBaseSize(QtCore.QSize(16, 16))
        self.loaderIconLabel.setText("")
        self.loaderIconLabel.setPixmap(QtGui.QPixmap(":/icons/loader"))
        self.loaderIconLabel.setScaledContents(True)
        self.loaderIconLabel.setAlignment(QtCore.Qt.AlignCenter)
        self.loaderIconLabel.setObjectName("loaderIconLabel")
        self.gridLayout.addWidget(self.loaderIconLabel, 1, 2, 1, 1)
        self.verticalLayout_2.addLayout(self.gridLayout)
        self.validatorNodeGroupBox = QtWidgets.QGroupBox(SmithWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.validatorNodeGroupBox.sizePolicy().hasHeightForWidth())
        self.validatorNodeGroupBox.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.validatorNodeGroupBox.setFont(font)
        self.validatorNodeGroupBox.setAutoFillBackground(False)
        self.validatorNodeGroupBox.setObjectName("validatorNodeGroupBox")
        self.formLayout = QtWidgets.QFormLayout(self.validatorNodeGroupBox)
        self.formLayout.setLabelAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.formLayout.setObjectName("formLayout")
        self.urlLabel = QtWidgets.QLabel(self.validatorNodeGroupBox)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.urlLabel.setFont(font)
        self.urlLabel.setObjectName("urlLabel")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.urlLabel)
        self.sessionKeysLabel = QtWidgets.QLabel(self.validatorNodeGroupBox)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.sessionKeysLabel.setFont(font)
        self.sessionKeysLabel.setObjectName("sessionKeysLabel")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.sessionKeysLabel)
        self.urlValueLabel = QtWidgets.QLabel(self.validatorNodeGroupBox)
        self.urlValueLabel.setText("")
        self.urlValueLabel.setObjectName("urlValueLabel")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.urlValueLabel)
        self.rotateKeysButton = QtWidgets.QPushButton(self.validatorNodeGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.rotateKeysButton.sizePolicy().hasHeightForWidth())
        self.rotateKeysButton.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setBold(False)
        font.setWeight(50)
        self.rotateKeysButton.setFont(font)
        self.rotateKeysButton.setObjectName("rotateKeysButton")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.FieldRole, self.rotateKeysButton)
        self.publishKeysButton = QtWidgets.QPushButton(self.validatorNodeGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.publishKeysButton.sizePolicy().hasHeightForWidth())
        self.publishKeysButton.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setBold(False)
        font.setWeight(50)
        self.publishKeysButton.setFont(font)
        self.publishKeysButton.setObjectName("publishKeysButton")
        self.formLayout.setWidget(3, QtWidgets.QFormLayout.FieldRole, self.publishKeysButton)
        self.sessionKeysTextBrowser = QtWidgets.QTextBrowser(self.validatorNodeGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.sessionKeysTextBrowser.sizePolicy().hasHeightForWidth())
        self.sessionKeysTextBrowser.setSizePolicy(sizePolicy)
        self.sessionKeysTextBrowser.setMaximumSize(QtCore.QSize(16777215, 50))
        font = QtGui.QFont()
        font.setBold(False)
        font.setWeight(50)
        self.sessionKeysTextBrowser.setFont(font)
        self.sessionKeysTextBrowser.setObjectName("sessionKeysTextBrowser")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.sessionKeysTextBrowser)
        self.verticalLayout_2.addWidget(self.validatorNodeGroupBox)
        self.errorLabel = QtWidgets.QLabel(SmithWidget)
        self.errorLabel.setStyleSheet("color: red;")
        self.errorLabel.setText("")
        self.errorLabel.setAlignment(QtCore.Qt.AlignCenter)
        self.errorLabel.setObjectName("errorLabel")
        self.verticalLayout_2.addWidget(self.errorLabel)
        self.acceptInvitationButton = QtWidgets.QPushButton(SmithWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.acceptInvitationButton.sizePolicy().hasHeightForWidth())
        self.acceptInvitationButton.setSizePolicy(sizePolicy)
        self.acceptInvitationButton.setObjectName("acceptInvitationButton")
        self.verticalLayout_2.addWidget(self.acceptInvitationButton)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.inviteButton = QtWidgets.QPushButton(SmithWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.inviteButton.sizePolicy().hasHeightForWidth())
        self.inviteButton.setSizePolicy(sizePolicy)
        self.inviteButton.setObjectName("inviteButton")
        self.horizontalLayout_2.addWidget(self.inviteButton)
        self.inviteAccountComboBox = QtWidgets.QComboBox(SmithWidget)
        self.inviteAccountComboBox.setObjectName("inviteAccountComboBox")
        self.horizontalLayout_2.addWidget(self.inviteAccountComboBox)
        self.verticalLayout_2.addLayout(self.horizontalLayout_2)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.certifyButton = QtWidgets.QPushButton(SmithWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.certifyButton.sizePolicy().hasHeightForWidth())
        self.certifyButton.setSizePolicy(sizePolicy)
        self.certifyButton.setObjectName("certifyButton")
        self.horizontalLayout.addWidget(self.certifyButton)
        self.certifySmithComboBox = QtWidgets.QComboBox(SmithWidget)
        self.certifySmithComboBox.setObjectName("certifySmithComboBox")
        self.horizontalLayout.addWidget(self.certifySmithComboBox)
        self.verticalLayout_2.addLayout(self.horizontalLayout)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.certifiersGroupBox = QtWidgets.QGroupBox(SmithWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.certifiersGroupBox.sizePolicy().hasHeightForWidth())
        self.certifiersGroupBox.setSizePolicy(sizePolicy)
        self.certifiersGroupBox.setObjectName("certifiersGroupBox")
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout(self.certifiersGroupBox)
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.certifiersListWidget = QtWidgets.QListWidget(self.certifiersGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.certifiersListWidget.sizePolicy().hasHeightForWidth())
        self.certifiersListWidget.setSizePolicy(sizePolicy)
        self.certifiersListWidget.setObjectName("certifiersListWidget")
        self.horizontalLayout_4.addWidget(self.certifiersListWidget)
        self.horizontalLayout_3.addWidget(self.certifiersGroupBox)
        self.certifiedGroupBox = QtWidgets.QGroupBox(SmithWidget)
        self.certifiedGroupBox.setObjectName("certifiedGroupBox")
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout(self.certifiedGroupBox)
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.certifiedListWidget = QtWidgets.QListWidget(self.certifiedGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.certifiedListWidget.sizePolicy().hasHeightForWidth())
        self.certifiedListWidget.setSizePolicy(sizePolicy)
        self.certifiedListWidget.setObjectName("certifiedListWidget")
        self.horizontalLayout_5.addWidget(self.certifiedListWidget)
        self.horizontalLayout_3.addWidget(self.certifiedGroupBox)
        self.verticalLayout_2.addLayout(self.horizontalLayout_3)

        self.retranslateUi(SmithWidget)
        QtCore.QMetaObject.connectSlotsByName(SmithWidget)
        SmithWidget.setTabOrder(self.accountComboBox, self.refreshSmithButton)
        SmithWidget.setTabOrder(self.refreshSmithButton, self.rotateKeysButton)
        SmithWidget.setTabOrder(self.rotateKeysButton, self.publishKeysButton)
        SmithWidget.setTabOrder(self.publishKeysButton, self.sessionKeysTextBrowser)

    def retranslateUi(self, SmithWidget):
        #  _translate = QtCore.QCoreApplication.translate
        SmithWidget.setWindowTitle(_("Smith"))
        self.label.setText(_("Select an account with a validated identity and a wallet"))
        self.goOnlineButton.setText(_("Go online"))
        self.accountLabel.setText(_("Account"))
        self.membershipLabel.setText(_("Smith Membership"))
        self.refreshSmithButton.setToolTip(_("Refresh nodes from network"))
        self.refreshSmithButton.setText(_("Refresh"))
        self.authorityLabel.setText(_("Authority"))
        self.goOfflineButton.setText(_("Go offline"))
        self.validatorNodeGroupBox.setTitle(_("Validator node"))
        self.urlLabel.setText(_("Url"))
        self.sessionKeysLabel.setText(_("Session keys"))
        self.rotateKeysButton.setText(_("Rotate keys"))
        self.publishKeysButton.setText(_("Publish keys"))
        self.acceptInvitationButton.setText(_("Accept invitation"))
        self.inviteButton.setText(_("Invite Member"))
        self.certifyButton.setText(_("Certify"))
        self.certifiersGroupBox.setTitle(_("Certifiers"))
        self.certifiedGroupBox.setTitle(_("Certified"))
import tikka.slots.pyqt.resources.icons.index_rc



# Copyright 2021 Vincent Texier <vit@free.fr>
#
# This software is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
from typing import List, Optional

from substrateinterface import Keypair

from tikka.domains.entities.identity import Certification, Identity, IdentityStatus
from tikka.interfaces.adapters.network.identities import NetworkIdentitiesInterface
from tikka.interfaces.adapters.repository.identities import (
    IdentitiesRepositoryInterface,
)


class Identities:

    """
    Identities domain class
    """

    def __init__(
        self,
        repository: IdentitiesRepositoryInterface,
        network: NetworkIdentitiesInterface,
    ):
        """
        Init Accounts domain

        :param repository: AccountsRepositoryInterface instance
        :param network: NetworkAccountsInterface instance
        """
        self.repository = repository
        self.network = network

    @staticmethod
    def create(
        index: int,
        removable_on: int,
        next_creatable_on: int,
        address: str,
        old_address: Optional[str],
        status: IdentityStatus = IdentityStatus.UNCONFIRMED,
        first_eligible_ud: int = 0,
    ):
        """
        Return an identity instance from params

        :param index: Index number in blockchain
        :param removable_on: Identity expiration timestamp
        :param next_creatable_on: Date after which a new identity can be created
        :param address: Account address
        :param old_address: Previous account address
        :param status: Identity status
        :param first_eligible_ud: First elligible UD index
        :return:
        """
        return Identity(
            index=index,
            removable_on=removable_on,
            next_creatable_on=next_creatable_on,
            status=status,
            address=address,
            old_address=old_address,
            first_eligible_ud=first_eligible_ud,
        )

    def add(self, identity: Identity):
        """
        Add identity in repository

        :param identity: Identity instance
        :return:
        """
        self.repository.add(identity)

    def update(self, identity: Identity):
        """
        Update identity in repository

        :param identity: Identity instance
        :return:
        """
        self.repository.update(identity)

    def get(self, index: int) -> Optional[Identity]:
        """
        Get identity instance

        :param index: Identity index
        :return:
        """
        return self.repository.get(index)

    def get_by_address(self, address: str) -> Optional[Identity]:
        """
        Return Identity instance from account address or None

        :param address: Account address
        :return:
        """
        return self.repository.get_by_address(address)

    def get_index_by_address(self, address: str) -> Optional[int]:
        """
        Return identity index from account address or None

        :param address: Account address
        :return:
        """
        return self.repository.get_index_by_address(address)

    def delete(self, index: int) -> None:
        """
        Delete identity in repository

        :param index: Identity index to delete
        :return:
        """
        self.repository.delete(index)

    def exists(self, index: int) -> bool:
        """
        Return True if identity exists in repository

        :param index: Identity index to check
        :return:
        """
        return self.repository.exists(index)

    def is_validated(self, index: int) -> bool:
        """
        Return True if identity status is validated

        :param index: Identity index to check
        :return:
        """
        identity = self.get(index)
        if identity is None:
            return False
        return identity.status == IdentityStatus.MEMBER

    def network_get_index(self, address: str) -> Optional[int]:
        """
        Get account identity index from network if any

        :param address: Account address
        :return:
        """
        identity_index = self.network.get_identity_index(address)
        if identity_index is None:
            old_identity_index = self.get_index_by_address(address)
            if old_identity_index is not None:
                self.delete(old_identity_index)

        return identity_index

    def network_get_identity_indice(self, addresses: List[str]) -> List[Optional[int]]:
        """
        Get account identity indice from current EntryPoint connection

        :param addresses: List of Account addresses
        :return:
        """
        identity_indice = self.network.get_identity_indice(addresses)
        # todo: purge smiths db from if index
        # todo: purge authorities db if None index
        for index, address in enumerate(addresses):
            if identity_indice[index] is None:
                identity_index = self.get_index_by_address(address)
                if identity_index is not None:
                    self.delete(identity_index)

        return identity_indice

    def network_get_identity(self, index: int) -> Optional[Identity]:
        """
        Get Identity instance by index from network if any

        :param index: Identity index
        :return:
        """
        identity = self.network.get_identity(index)
        if identity is not None:
            if self.exists(index) is True:
                self.update(identity)
            else:
                self.add(identity)

        return identity

    def network_get_identities(self, identity_indice: List[int]) -> None:
        """
        Get Identity instances by index list from network

        :param identity_indice: Identity index list
        :return:
        """
        identities = self.network.get_identities(identity_indice)
        for index, identity in enumerate(identities):
            if identity is None:
                self.repository.delete(identity_indice[index])
                continue
            if self.exists(identity.index) is True:
                self.update(identity)
            else:
                self.add(identity)

    def network_change_owner_key(
        self, old_keypair: Keypair, new_keypair: Keypair
    ) -> None:
        """
        Change identity owner from old_keypair to new_keypair on blockchain

        :param old_keypair: Keypair of current identity account
        :param new_keypair: Keypair of new identity account
        :return:
        """
        return self.network.change_owner_key(old_keypair, new_keypair)

    def network_get_certs_by_receiver(
        self, receiver_address: str, receiver_identity_index: int
    ) -> Optional[List[Certification]]:
        """
        Get certification (identity index, expire on block number) list for identity index from network if any

        :param receiver_address: Address of receiver account
        :param receiver_identity_index: Identity index of receiver
        :return:
        """
        return self.network.certs_by_receiver(receiver_address, receiver_identity_index)

    def network_claim_uds(self, keypair: Keypair) -> None:
        """
        Add unclaimed UDs of identity to keypair account balance

        :param keypair: Keypair of account
        :return:
        """
        return self.network.claim_uds(keypair)



# Copyright 2021 Vincent Texier <vit@free.fr>
#
# This software is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import abc

from tikka.interfaces.domains.connections import ConnectionsInterface


class NetworkCurrencyInterface(abc.ABC):
    """
    NetworkCurrencyInterface class
    """

    def __init__(self, connections: ConnectionsInterface) -> None:
        """
        Use connections to request currency informations

        :param connections: ConnectionsInterface instance
        :return:
        """
        self.connections = connections

    @abc.abstractmethod
    def get_token_decimals(self) -> int:
        """
        Return the number of decimals for the token storage
        ex: 2 = store centimes, 1 token = stored value / 100

        :return:
        """
        raise NotImplementedError

    @abc.abstractmethod
    def get_token_symbol(self) -> str:
        """
        Return the token symbol of the currency

        :return:
        """
        raise NotImplementedError

    @abc.abstractmethod
    def get_universal_dividend(self) -> int:
        """
        Return the universal dividend current amount

        :return:
        """
        raise NotImplementedError

    @abc.abstractmethod
    def get_monetary_mass(self) -> int:
        """
        Return the monetary mass current amount

        :return:
        """
        raise NotImplementedError

    @abc.abstractmethod
    def get_members_count(self) -> int:
        """
        Return the monetary mass current amount

        :return:
        """
        raise NotImplementedError

    @abc.abstractmethod
    def get_expected_block_time(self) -> int:
        """
        Return the expected block time in ms

        :return:
        """
        raise NotImplementedError

    @abc.abstractmethod
    def get_epoch_duration(self) -> int:
        """
        Return the expected session time in ms

        :return:
        """
        raise NotImplementedError


class NetworkCurrencyException(Exception):
    """
    NetworkCurrencyException class
    """



# Copyright 2021 Vincent Texier <vit@free.fr>
#
# This software is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
import logging
import struct
from typing import List, Optional

from substrateinterface import Keypair
from substrateinterface.exceptions import SubstrateRequestException

from tikka.domains.entities.identity import Certification, Identity, IdentityStatus
from tikka.interfaces.adapters.network.connection import NetworkConnectionError
from tikka.interfaces.adapters.network.identities import (
    NetworkIdentitiesException,
    NetworkIdentitiesInterface,
)


class NetworkIdentities(NetworkIdentitiesInterface):
    """
    NetworkIdentities class
    """

    status_map = {
        "Unconfirmed": IdentityStatus.UNCONFIRMED,
        "Unvalidated": IdentityStatus.UNVALIDATED,
        "Member": IdentityStatus.MEMBER,
        "NotMember": IdentityStatus.NOT_MEMBER,
        "Revoked": IdentityStatus.REVOKED,
    }

    def get_identity_index(self, address: str) -> Optional[int]:
        __doc__ = (  # pylint: disable=redefined-builtin, unused-variable
            NetworkIdentitiesInterface.get_identity_index.__doc__
        )
        if not self.connections.is_connected() or self.connections.rpc.client is None:
            raise NetworkIdentitiesException(NetworkConnectionError())

        try:
            result = self.connections.rpc.client.query(
                "Identity", "IdentityIndexOf", [address]
            )
        except Exception as exception:
            logging.exception(exception)
            raise NetworkIdentitiesException(exception)

        return result.value

    def get_identity_indice(self, addresses: List[str]) -> List[Optional[int]]:
        __doc__ = (  # pylint: disable=redefined-builtin, unused-variable
            NetworkIdentitiesInterface.get_identity_indice.__doc__
        )
        if not self.connections.is_connected() or self.connections.rpc.client is None:
            raise NetworkIdentitiesException(NetworkConnectionError())

        storage_keys = []
        for address in addresses:
            storage_keys.append(
                self.connections.rpc.client.create_storage_key(
                    "Identity", "IdentityIndexOf", [address]
                )
            )

        try:
            multi_result = self.connections.rpc.client.query_multi(storage_keys)
        except Exception as exception:
            logging.exception(exception)
            raise NetworkIdentitiesException(exception)

        identity_indice = []
        for index, (storage_key, value_obj) in enumerate(multi_result):
            identity_indice.append(value_obj.value)

        return identity_indice

    def get_identity(self, index: int) -> Optional[Identity]:
        __doc__ = (  # pylint: disable=redefined-builtin, unused-variable
            NetworkIdentitiesInterface.get_identity.__doc__
        )
        if not self.connections.is_connected() or self.connections.rpc.client is None:
            raise NetworkIdentitiesException(NetworkConnectionError())

        try:
            result = self.connections.rpc.client.query(
                "Identity", "Identities", [index]
            )
        except Exception as exception:
            logging.exception(exception)
            raise NetworkIdentitiesException(exception)

        if result is None:
            return None
        old_address = result["old_owner_key"].value
        if old_address is not None:
            old_address = old_address[0]
        return Identity(
            index=index,
            next_creatable_on=result["next_creatable_identity_on"].value,
            removable_on=int(result["next_scheduled"].value),
            status=self.status_map[result["status"].value],
            address=result["owner_key"].value,
            old_address=old_address,
            first_eligible_ud=result.value["data"]["first_eligible_ud"],
        )

    def get_identities(self, identity_indice: List[int]) -> List[Optional[Identity]]:
        __doc__ = (  # pylint: disable=redefined-builtin, unused-variable
            NetworkIdentitiesInterface.get_identities.__doc__
        )
        if not self.connections.is_connected() or self.connections.rpc.client is None:
            raise NetworkIdentitiesException(NetworkConnectionError())

        storage_keys = []
        for identity_index in identity_indice:
            storage_keys.append(
                self.connections.rpc.client.create_storage_key(
                    "Identity", "Identities", [identity_index]
                )
            )

        try:
            multi_result = self.connections.rpc.client.query_multi(storage_keys)
        except Exception as exception:
            logging.exception(exception)
            raise NetworkIdentitiesException(exception)

        # def: {
        #   Composite: {
        #     fields: [
        #       {
        #         name: data
        #         # type: 268
        #         typeName: IdtyData
        #         docs: []
        #       }
        #       {
        #         name: next_creatable_identity_on
        #         # type: 4
        #         typeName: BlockNumber
        #         docs: []
        #       }
        #       {
        #         name: old_owner_key
        #         # type: 269
        #         typeName: Option<(AccountId, BlockNumber)>
        #         docs: []
        #       }
        #       {
        #         name: owner_key
        #         # type: 0
        #         typeName: AccountId
        #         docs: []
        #       }
        #       {
        #         name: next_scheduled
        #         # type: 4
        #         typeName: BlockNumber
        #         docs: []
        #       }
        #       {
        #         name: status
        #         # type: 271
        #         typeName: IdtyStatus
        #         docs: []
        #       }
        #     ]
        #   }
        # }

        identities: List[Optional[Identity]] = []
        for index, (storage_key, value_obj) in enumerate(multi_result):
            if value_obj.value is not None:
                old_address_value = value_obj["old_owner_key"].value
                old_address = None
                if old_address_value is not None:
                    if isinstance(old_address_value, tuple):
                        old_address = old_address_value[0]
                    else:
                        old_address = old_address_value

                identities.append(
                    Identity(
                        index=storage_keys[index].params[0],
                        next_creatable_on=value_obj["next_creatable_identity_on"].value,
                        removable_on=int(value_obj["next_scheduled"].value),
                        status=self.status_map[value_obj["status"].value],
                        address=value_obj["owner_key"].value,
                        old_address=old_address,
                        first_eligible_ud=value_obj.value["data"]["first_eligible_ud"],
                    )
                )
            else:
                identities.append(None)

        return identities

    def change_owner_key(self, old_keypair: Keypair, new_keypair: Keypair) -> None:
        __doc__ = (  # pylint: disable=redefined-builtin, unused-variable
            NetworkIdentitiesInterface.change_owner_key.__doc__
        )
        if not self.connections.is_connected() or self.connections.rpc.client is None:
            raise NetworkIdentitiesException(NetworkConnectionError())

        identity_index = self.get_identity_index(old_keypair.ss58_address)
        if identity_index is None:
            raise NetworkIdentitiesException("No identity found for origin account")

        # message to sign
        prefix_bytes = b"icok"
        genesis_hash_str = self.connections.rpc.client.get_block_hash(0)
        genesis_hash_bytes = bytearray.fromhex(genesis_hash_str[2:])
        identity_index_bytes = struct.pack("<I", identity_index)
        identity_pubkey_bytes = old_keypair.public_key
        message_bytes = (
            prefix_bytes
            + genesis_hash_bytes
            + identity_index_bytes
            + identity_pubkey_bytes
        )

        # message signed by the new owner
        signature_bytes = new_keypair.sign(message_bytes)

        # newKey: AccountId32, newKeySig: SpRuntimeMultiSignature
        params = {
            "new_key": new_keypair.ss58_address,
            "new_key_sig": {"Sr25519": signature_bytes},
        }
        try:
            # create raw call (extrinsic)
            call = self.connections.rpc.client.compose_call(
                call_module="Identity",
                call_function="change_owner_key",
                call_params=params,
            )
        except Exception as exception:
            logging.exception(exception)
            raise NetworkIdentitiesException(exception)

        try:
            # create extrinsic signed by current owner
            extrinsic = self.connections.rpc.client.create_signed_extrinsic(
                call=call, keypair=old_keypair
            )
        except Exception as exception:
            logging.exception(exception)
            raise NetworkIdentitiesException(exception)

        try:
            result = self.connections.rpc.client.submit_extrinsic(
                extrinsic, wait_for_inclusion=True
            )
        except SubstrateRequestException as exception:
            logging.exception(exception)
            raise NetworkIdentitiesException(exception)

        if result.is_success is False:
            logging.error(result.error_message)
            raise NetworkIdentitiesException(result.error_message["name"])

    def certs_by_receiver(
        self, receiver_address: str, receiver_identity_index: int
    ) -> List[Certification]:
        __doc__ = (  # pylint: disable=redefined-builtin, unused-variable
            NetworkIdentitiesInterface.certs_by_receiver.__doc__
        )
        if not self.connections.is_connected() or self.connections.rpc.client is None:
            raise NetworkIdentitiesException(NetworkConnectionError())

        try:
            result = self.connections.rpc.client.query(
                "Certification", "CertsByReceiver", [receiver_identity_index]
            )
        except Exception as exception:
            logging.exception(exception)
            raise NetworkIdentitiesException(exception)

        storage_keys = []
        for issuer_identity_index, cert_expire_on_block in result.value:
            storage_keys.append(
                self.connections.rpc.client.create_storage_key(
                    "Identity", "Identities", [issuer_identity_index]
                )
            )

        multi_result = self.connections.rpc.client.query_multi(storage_keys)

        certifications = []
        for index, (storage_key, value_obj) in enumerate(multi_result):
            certifications.append(
                Certification(
                    issuer_identity_index=storage_keys[index].params[0],
                    issuer_address=value_obj.value["owner_key"],
                    receiver_identity_index=receiver_identity_index,
                    receiver_address=receiver_address,
                    expire_on_block=result.value[index][1],
                )
            )

        return certifications

    def claim_uds(self, keypair: Keypair) -> None:
        __doc__ = (  # pylint: disable=redefined-builtin, unused-variable
            NetworkIdentitiesInterface.claim_uds.__doc__
        )
        if not self.connections.is_connected() or self.connections.rpc.client is None:
            raise NetworkIdentitiesException(NetworkConnectionError())

        try:
            # create raw call (extrinsic)
            call = self.connections.rpc.client.compose_call(
                call_module="UniversalDividend",
                call_function="claim_uds",
            )
        except Exception as exception:
            logging.exception(exception)
            raise NetworkIdentitiesException(exception)

        try:
            # create extrinsic signed by current owner
            extrinsic = self.connections.rpc.client.create_signed_extrinsic(
                call=call, keypair=keypair
            )
        except Exception as exception:
            logging.exception(exception)
            raise NetworkIdentitiesException(exception)

        try:
            print("send claim uds...")
            result = self.connections.rpc.client.submit_extrinsic(
                extrinsic, wait_for_inclusion=True
            )
        except SubstrateRequestException as exception:
            logging.exception(exception)
            raise NetworkIdentitiesException(exception)

        if result.is_success is False:
            logging.error(result.error_message)
            raise NetworkIdentitiesException(result.error_message["name"])



# Copyright 2021 Vincent Texier <vit@free.fr>
#
# This software is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from typing import List, Optional

from tikka.adapters.repository.sqlite3 import Sqlite3RepositoryInterface
from tikka.domains.entities.wallet import Wallet
from tikka.interfaces.adapters.repository.wallets import WalletsRepositoryInterface

TABLE_NAME = "wallets"


class Sqlite3WalletsRepository(WalletsRepositoryInterface, Sqlite3RepositoryInterface):
    """
    Sqlite3WalletsRepository class
    """

    def list(self) -> List[Wallet]:
        __doc__ = (  # pylint: disable=redefined-builtin, unused-variable
            WalletsRepositoryInterface.list.__doc__
        )

        result_set = self.client.select(f"SELECT * FROM {TABLE_NAME}")

        list_ = []
        for row in result_set:
            list_.append(Wallet(*row))

        return list_

    def list_addresses(self) -> List[str]:
        __doc__ = (  # pylint: disable=redefined-builtin, unused-variable
            WalletsRepositoryInterface.list_addresses.__doc__
        )

        result_set = self.client.select(f"SELECT address FROM {TABLE_NAME}")

        list_ = []
        for row in result_set:
            list_.append(row[0])

        return list_

    def add(self, wallet: Wallet) -> None:
        __doc__ = (  # pylint: disable=redefined-builtin, unused-variable
            WalletsRepositoryInterface.add.__doc__
        )

        # insert only non hidden fields
        self.client.insert(
            TABLE_NAME,
            **{
                key: value
                for (key, value) in wallet.__dict__.items()
                if not key.startswith("_")
            },
        )

    def get(self, address: str) -> Optional[Wallet]:
        __doc__ = (  # pylint: disable=redefined-builtin, unused-variable
            WalletsRepositoryInterface.get.__doc__
        )

        row = self.client.select_one(
            f"SELECT * FROM {TABLE_NAME} WHERE address=?", (address,)
        )
        if row is None:
            return None

        return Wallet(*row)

    def update(self, wallet: Wallet) -> None:
        __doc__ = (  # pylint: disable=redefined-builtin, unused-variable
            WalletsRepositoryInterface.update.__doc__
        )

        # update only non hidden fields
        self.client.update(
            TABLE_NAME,
            f"address='{wallet.address}'",
            **{
                key: value
                for (key, value) in wallet.__dict__.items()
                if not key.startswith("_")
            },
        )

    def delete(self, address: str) -> None:
        __doc__ = (  # pylint: disable=redefined-builtin, unused-variable
            WalletsRepositoryInterface.delete.__doc__
        )

        self.client.delete(TABLE_NAME, address=address)

    def exists(self, address: str) -> bool:
        __doc__ = (  # pylint: disable=redefined-builtin, unused-variable
            WalletsRepositoryInterface.exists.__doc__
        )

        row = self.client.select_one(
            f"SELECT count(address) FROM {TABLE_NAME} WHERE address=?", (address,)
        )
        if row is None:
            return False

        return row[0] == 1



# Copyright 2021 Vincent Texier <vit@free.fr>
#
# This software is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import gettext
import logging
from pathlib import Path
from time import sleep
from typing import Any

from tikka.adapters.network.accounts import NetworkAccounts
from tikka.adapters.network.authorities import NetworkAuthorities
from tikka.adapters.network.currency import NetworkCurrency
from tikka.adapters.network.identities import NetworkIdentities
from tikka.adapters.network.nodes import NetworkNodes
from tikka.adapters.network.rpc.connection import RPCConnection
from tikka.adapters.network.smiths import NetworkSmiths
from tikka.adapters.network.technical_committee import NetworkTechnicalCommittee
from tikka.adapters.network.transfers import NetworkTransfers
from tikka.adapters.repository.accounts import Sqlite3AccountsRepository
from tikka.adapters.repository.authorities import Sqlite3AuthoritiesRepository
from tikka.adapters.repository.categories import Sqlite3CategoriesRepository
from tikka.adapters.repository.config import FileConfigRepository
from tikka.adapters.repository.currencies import FileCurrenciesRepository
from tikka.adapters.repository.currency import Sqlite3CurrencyRepository
from tikka.adapters.repository.file_wallets import V1FileWalletsRepository
from tikka.adapters.repository.identities import Sqlite3IdentitiesRepository
from tikka.adapters.repository.nodes import Sqlite3NodesRepository
from tikka.adapters.repository.passwords import Sqlite3PasswordsRepository
from tikka.adapters.repository.preferences import Sqlite3PreferencesRepository
from tikka.adapters.repository.smiths import Sqlite3SmithsRepository
from tikka.adapters.repository.sqlite3 import Sqlite3Client
from tikka.adapters.repository.tabs import Sqlite3TabRepository
from tikka.adapters.repository.wallets import Sqlite3WalletsRepository
from tikka.domains.accounts import Accounts
from tikka.domains.amounts import Amounts
from tikka.domains.authorities import Authorities
from tikka.domains.categories import Categories
from tikka.domains.config import Config
from tikka.domains.connections import Connections
from tikka.domains.currencies import Currencies
from tikka.domains.entities.constants import LOCALES_PATH
from tikka.domains.entities.events import CurrencyEvent
from tikka.domains.events import EventDispatcher
from tikka.domains.identities import Identities
from tikka.domains.nodes import Nodes
from tikka.domains.passwords import Passwords
from tikka.domains.preferences import Preferences
from tikka.domains.smiths import Smiths
from tikka.domains.technical_committee import TechnicalCommittee
from tikka.domains.transfers import Transfers
from tikka.domains.vaults import Vaults
from tikka.domains.wallets import Wallets
from tikka.interfaces.adapters.network.currency import NetworkCurrencyException


class Application:
    """
    Application class
    """

    def __init__(self, data_path: Path):
        """
        Init application

        :param data_path: Path instance of application data folder
        """

        # data
        self.data_path = data_path
        # dependency injection
        # init event dispatcher
        self.event_dispatcher = EventDispatcher()
        # init network connection to node
        self.connections = Connections(RPCConnection(), self.event_dispatcher)

        # init currencies support list
        currencies_repository = FileCurrenciesRepository()

        # init config domain
        config_repository = FileConfigRepository(self.data_path)
        self.config = Config(config_repository)

        # if supported currencies has changed and config use an unknown currency...
        if (
            self.config.get(Config.CURRENCY_KEY)
            not in currencies_repository.code_names()
        ):
            # set current currency to first currency by default
            self.config.set(Config.CURRENCY_KEY, currencies_repository.code_names()[0])

        # init wallets adapter
        self.file_wallets_repository = V1FileWalletsRepository()
        # database adapter
        self.sqlite3_client = Sqlite3Client(
            self.config.get(Config.CURRENCY_KEY), self.data_path
        )
        # init SQL repositories
        accounts_repository = Sqlite3AccountsRepository(self.sqlite3_client)
        wallets_repository = Sqlite3WalletsRepository(self.sqlite3_client)
        identities_repository = Sqlite3IdentitiesRepository(self.sqlite3_client)
        smiths_repository = Sqlite3SmithsRepository(self.sqlite3_client)
        authorities_repository = Sqlite3AuthoritiesRepository(self.sqlite3_client)
        nodes_repository = Sqlite3NodesRepository(self.sqlite3_client)
        currency_repository = Sqlite3CurrencyRepository(self.sqlite3_client)
        categories_repository = Sqlite3CategoriesRepository(self.sqlite3_client)
        passwords_repository = Sqlite3PasswordsRepository(self.sqlite3_client)
        self.tab_repository = Sqlite3TabRepository(self.sqlite3_client)
        self.preferences_repository = Sqlite3PreferencesRepository(self.sqlite3_client)

        # init translation
        self.translator = self.init_i18n()

        # init domains
        self.preferences = Preferences(self.preferences_repository)
        self.currencies = Currencies(
            currencies_repository,
            currency_repository,
            NetworkCurrency(self.connections),
            self.config.get(Config.CURRENCY_KEY),
        )
        self.passwords = Passwords(passwords_repository)
        self.wallets = Wallets(wallets_repository, self.currencies)
        self.identities = Identities(
            identities_repository, NetworkIdentities(self.connections)
        )
        self.accounts = Accounts(
            accounts_repository,
            NetworkAccounts(self.connections),
            self.passwords,
            self.wallets,
            self.file_wallets_repository,
            self.currencies,
            self.event_dispatcher,
        )
        self.amounts = Amounts(self.currencies, self.translator)
        self.transfers = Transfers(
            self.wallets, NetworkTransfers(self.connections), self.event_dispatcher
        )
        self.nodes = Nodes(
            nodes_repository,
            self.preferences,
            self.connections,
            NetworkNodes(self.connections),
            self.config,
            self.currencies,
            self.event_dispatcher,
        )
        self.smiths = Smiths(smiths_repository, NetworkSmiths(self.connections))
        self.authorities = Authorities(
            authorities_repository,
            NetworkAuthorities(self.connections),
            self.nodes,
            self.smiths,
        )
        self.categories = Categories(
            categories_repository, self.accounts, self.event_dispatcher
        )
        self.technical_committee = TechnicalCommittee(
            NetworkTechnicalCommittee(self.connections), self.event_dispatcher
        )
        self.vaults = Vaults(
            NetworkAccounts(self.connections),
            self.accounts,
            self.currencies,
        )

        # init network connections
        if self.config.get(Config.RANDOM_CONNECTION_AT_START_KEY) is True:
            self.nodes.set_current_url_randomly()
        else:
            current_node = self.nodes.get(self.nodes.get_current_url())
            if current_node is not None:
                # connect to RPC API via websocket
                self.connections.connect(current_node)

        # if currency properties required for amount display not populated...
        if self.currencies.get_current().members_count is None:
            try:
                # fetch currency properties from network
                self.currencies.network_get_properties()
            except NetworkCurrencyException:
                self.currencies.get_current().members_count = 1

    def init_i18n(self) -> Any:
        """
        Init translator from configured language

        :return:
        """
        # define translator for configurated language
        translator = gettext.translation(
            "application",
            str(LOCALES_PATH),
            languages=[self.config.get(Config.LANGUAGE_KEY)],
        )
        # init translator
        translator.install()

        return translator

    def select_currency(self, code_name: str):
        """
        Change currency

        :return:
        """
        if self.config is None:
            raise NoConfigError

        # dispatch event EVENT_TYPE_CHANGED
        event = CurrencyEvent(CurrencyEvent.EVENT_TYPE_PRE_CHANGE, code_name)
        self.event_dispatcher.dispatch_event(event)

        self.config.set(Config.CURRENCY_KEY, code_name)

        if self.sqlite3_client is not None:
            self.sqlite3_client.close()

        while self.sqlite3_client.is_alive():
            sleep(1)
            logging.debug("wait sqlite3 thread to end...")

        # init database connection
        self.sqlite3_client = Sqlite3Client(code_name, self.data_path)

        while not self.sqlite3_client.is_alive():
            sleep(1)
            logging.debug("wait sqlite3 thread to start...")

        # create new repository adapters on the new database
        self.accounts.repository.set_client(self.sqlite3_client)  # type: ignore
        self.passwords.repository.set_client(self.sqlite3_client)  # type: ignore
        self.wallets.repository.set_client(self.sqlite3_client)  # type: ignore
        self.identities.repository.set_client(self.sqlite3_client)  # type: ignore
        self.smiths.repository.set_client(self.sqlite3_client)  # type: ignore
        self.authorities.repository.set_client(self.sqlite3_client)  # type: ignore
        self.tab_repository.set_client(self.sqlite3_client)
        self.preferences_repository.set_client(self.sqlite3_client)
        self.nodes.repository.set_client(self.sqlite3_client)  # type: ignore
        self.currencies.currency_repository.set_client(self.sqlite3_client)  # type: ignore
        self.categories.repository.set_client(self.sqlite3_client)  # type: ignore
        # fixme: domains should only access others domains, not repositories of other domains
        #        we should create a preferences domain (and tabs domain too)

        self.currencies.set_current(code_name)

        # init domains with new repository adapter
        self.nodes.init_repository()

        # get current entry point for new network connection
        current_node = self.nodes.get(self.nodes.get_current_url())
        if current_node is not None:
            # disconnect previous currency connection
            self.connections.disconnect()
            # connect to RPC API via websocket
            self.connections.connect(current_node)

        self.currencies.network_get_properties()

        # dispatch event EVENT_TYPE_CHANGED
        event = CurrencyEvent(
            CurrencyEvent.EVENT_TYPE_CHANGED, self.currencies.get_current().code_name
        )
        self.event_dispatcher.dispatch_event(event)

    def select_language(self, language: str):
        """
        Select GUI language

        :param language: Code of language (ex: "en_US", "fr_FR")
        :return:
        """
        if self.config is None:
            raise NoConfigError

        self.config.set(Config.LANGUAGE_KEY, language)
        self.translator = self.init_i18n()

    def close(self):
        """
        Quit application and close what needs to be closed

        :return:
        """
        # disconnect all connections
        self.connections.disconnect()
        # close Sqlite3 client thread
        self.sqlite3_client.close()


class NoDatabaseError(Exception):
    pass


class NoConfigError(Exception):
    pass



# Copyright 2021 Vincent Texier <vit@free.fr>
#
# This software is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# Copyright 2023 Vincent Texier <vit@free.fr>
#
# This software is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from typing import Any

from tikka.interfaces.adapters.repository.preferences import (
    PreferencesRepositoryInterface,
)


class Preferences:
    """
    Preferences domain class
    """

    def __init__(self, repository: PreferencesRepositoryInterface):
        """
        Init Preferences domain instance

        :param repository: PreferencesRepositoryInterface instance
        """
        self.repository = repository

    def set(self, key: str, value: Any):
        """
        Set key to value

        :param key: Key string
        :param value: Value
        :return:
        """
        self.repository.set(key, value)

    def get(self, key: str):
        """
        Return value of key

        :param key: Key string
        :return:
        """
        return self.repository.get(key)



# Copyright 2021 Vincent Texier <vit@free.fr>
#
# This software is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
import json
import logging
import random
from json import JSONDecodeError
from typing import List, Optional
from urllib import request

from tikka.adapters.network.nodes import NetworkNodes
from tikka.adapters.network.rpc.connection import RPCConnection
from tikka.domains.config import Config
from tikka.domains.connections import Connections
from tikka.domains.currencies import Currencies
from tikka.domains.entities.constants import (
    NODES_CURRENT_ENTRY_POINT_URL_PREFERENCES_KEY,
)
from tikka.domains.entities.events import ConnectionsEvent, NodesEvent
from tikka.domains.entities.node import Node
from tikka.domains.events import EventDispatcher
from tikka.domains.preferences import Preferences
from tikka.interfaces.adapters.network.nodes import NetworkNodesInterface
from tikka.interfaces.adapters.repository.nodes import NodesRepositoryInterface


class Nodes:
    """
    Nodes domain class
    """

    def __init__(
        self,
        repository: NodesRepositoryInterface,
        preferences: Preferences,
        connections: Connections,
        network: NetworkNodesInterface,
        config: Config,
        currencies: Currencies,
        event_dispatcher: EventDispatcher,
    ):
        """
        Init Nodes domain instance

        :param repository: EntryPointsRepositoryInterface instance
        :param preferences: Preferences domain instance
        :param connections: Connections instance
        :param network: Network adapter instance for handling nodes
        :param config: Config instance
        :param currencies: Currencies instance
        :param event_dispatcher: EventDispatcher instance
        """
        self.repository = repository
        self.preferences = preferences
        self.connections = connections
        self.network = network
        self.config = config
        self.currencies = currencies
        self.event_dispatcher = event_dispatcher
        self._current_url = self.currencies.get_entry_point_urls()["rpc"][0]

        self.init_repository()

        # events
        self.event_dispatcher.add_event_listener(
            ConnectionsEvent.EVENT_TYPE_CONNECTED, self._on_connections_connected
        )

    def init_repository(self):
        """
        Init repository with default entry points from config

        :return:
        """
        repository_urls = self.repository.get_urls()
        currency_endpoints = self.currencies.get_entry_point_urls()

        # init repository with current currency entry point urls
        for url in currency_endpoints["rpc"]:
            if url not in repository_urls:
                self.repository.add(Node(url))

        # fetch endpoints from online json file
        if currency_endpoints["online"]:
            try:
                with request.urlopen(currency_endpoints["online"]) as file:
                    online_endpoints = json.load(file)
            except JSONDecodeError as exception:
                logging.warning(
                    "error loading endpoints from %s", currency_endpoints["online"]
                )
                logging.exception(exception)
                online_endpoints = None

            if online_endpoints is not None:
                repository_urls = self.repository.get_urls()
                for url in online_endpoints["rpc"]:
                    if url not in repository_urls:
                        self.repository.add(Node(url))

        self._current_url = self.repository.list(0, 1)[0].url

        current_url_in_preferences = self.preferences.get(
            NODES_CURRENT_ENTRY_POINT_URL_PREFERENCES_KEY
        )
        if (
            current_url_in_preferences is None
            or current_url_in_preferences not in self.repository.get_urls()
        ):
            self.preferences.set(
                NODES_CURRENT_ENTRY_POINT_URL_PREFERENCES_KEY, self.get_current_url()
            )
        else:
            self._current_url = current_url_in_preferences

    def network_fetch_current_node(self) -> None:
        """
        Update node from network

        :return:
        """
        current_node = self.repository.get(self.get_current_url())
        network_node = self.network.get()
        if network_node is None:
            return None

        if current_node is not None:
            # update only changing properties
            current_node.block = network_node.block
            current_node.peer_id = network_node.peer_id
            current_node.software = network_node.software
            current_node.software_version = network_node.software_version
            current_node.epoch_index = network_node.epoch_index
            current_node.unsafe_api_exposed = network_node.unsafe_api_exposed

            self.repository.update(current_node)

        return None

    @staticmethod
    def network_test_and_get_node(url: str) -> Optional[Node]:
        """
        Try to open connection on url and return node if successful

        Then close connection

        :param url: Entry point url
        :return:
        """
        node = None

        connections = Connections(RPCConnection(), EventDispatcher())
        connections.connect(Node(url))
        if connections.is_connected():
            network_nodes = NetworkNodes(connections)
            node = network_nodes.get()
        connections.disconnect()

        return node

    def set_current_url_randomly(self):
        """
        Shuffle node list randomly and connect to first available node

        :return:
        """
        indices = list(range(1, self.count()))
        random.shuffle(indices)
        for index in indices:
            url = self.list()[index].url
            # never choose localhost randomly...
            if "localhost" not in url:
                self.set_current_url(self.list()[index].url)
                if self.connections.is_connected():
                    break

    def add(self, node: Node) -> None:
        """
        Add node in repository

        :param node: Node instance
        :return:
        """
        self.repository.add(node)

        self.event_dispatcher.dispatch_event(
            NodesEvent(NodesEvent.EVENT_TYPE_LIST_CHANGED)
        )

    def get(self, url: str) -> Optional[Node]:
        """
        Get Node instance by url

        :param url: Url
        :return:
        """
        return self.repository.get(url)

    def update(self, node: Node) -> None:
        """
        Update Node in repository

        :param node: Node instance
        :return:
        """
        self.repository.update(node)

        self.event_dispatcher.dispatch_event(
            NodesEvent(NodesEvent.EVENT_TYPE_LIST_CHANGED)
        )

    def list(self) -> List[Node]:
        """
        Return all Nodes from repository

        :return:
        """
        return self.repository.list()

    def count(self) -> int:
        """
        Return total node count

        :return:
        """
        return self.repository.count()

    def delete(self, url: str):
        """
        Delete Node by url

        :param url: Node url
        :return:
        """
        # do not delete default entry points from config
        if url in self.currencies.get_entry_point_urls()["rpc"]:
            return

        self.repository.delete(url)
        # switch current entry point to first in list
        self.set_current_url(self.repository.list(0, 1)[0].url)
        # set new entry point in preferences
        self.preferences.set(
            NODES_CURRENT_ENTRY_POINT_URL_PREFERENCES_KEY, self.get_current_url()
        )

        self.event_dispatcher.dispatch_event(
            NodesEvent(NodesEvent.EVENT_TYPE_LIST_CHANGED)
        )

    def get_current_url(self) -> str:
        """
        Return current entry point url

        :return:
        """
        return self._current_url

    def set_current_url(self, url: str) -> None:
        """
        Set current entry point url

        :return:
        """
        self._current_url = url
        # update preference
        self.preferences.set(
            NODES_CURRENT_ENTRY_POINT_URL_PREFERENCES_KEY,
            self._current_url,
        )
        # switch current connection
        self.connections.disconnect()
        node = self.get(self._current_url)
        if node is not None:
            self.connections.connect(node)

    def _on_connections_connected(self, _: ConnectionsEvent):
        """
        Triggered when the connection is established

        :param _:
        :return:
        """
        self.network_fetch_current_node()



# Copyright 2021 Vincent Texier <vit@free.fr>
#
# This software is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import abc
from typing import List

from substrateinterface import Keypair

from tikka.domains.entities.technical_committee import TechnicalCommitteeProposal
from tikka.interfaces.domains.connections import ConnectionsInterface


class NetworkTechnicalCommitteeInterface(abc.ABC):
    """
    NetworkTechnicalCommitteeInterface class
    """

    def __init__(self, connections: ConnectionsInterface) -> None:
        """
        Use connections to request/send Technical Committee information

        :param connections: ConnectionsInterface instance
        :return:
        """
        self.connections = connections

    @abc.abstractmethod
    def members(self) -> List[str]:
        """
        Return list of member addresses

        :return:
        """
        raise NotImplementedError

    @abc.abstractmethod
    def proposals(self) -> List[TechnicalCommitteeProposal]:
        """
        Return list of Smith instance from list of identity indice

        :return:
        """
        raise NotImplementedError

    @abc.abstractmethod
    def vote(
        self, keypair: Keypair, proposal: TechnicalCommitteeProposal, vote: bool
    ) -> None:
        """
        Send Technical Committee Vote for proposal from Keypair

        :param keypair: Keypair instance
        :param proposal: TechnicalCommitteeProposal instance
        :param vote: True or False
        :return:
        """
        raise NotImplementedError


class NetworkTechnicalCommitteeException(Exception):
    """
    NetworkTechnicalCommitteeException class
    """



# Copyright 2021 Vincent Texier <vit@free.fr>
#
# This software is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from dataclasses import dataclass, field
from datetime import datetime
from enum import Enum
from typing import List, Optional, TypeVar

SmithType = TypeVar("SmithType", bound="Smith")


class SmithStatus(int, Enum):
    INVITED = 0
    PENDING = 1
    SMITH = 2
    EXCLUDED = 3


@dataclass
class Smith:
    identity_index: int
    status: SmithStatus
    expire_on: Optional[datetime]
    certifications_received: List[int] = field(default_factory=lambda: list())
    certifications_issued: List[int] = field(default_factory=lambda: list())



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'tikka/slots/pyqt/resources/gui/windows/scan_qrcode.ui'
#
# Created by: PyQt5 UI code generator 5.15.9
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_ScanQRCodeDialog(object):
    def setupUi(self, ScanQRCodeDialog):
        ScanQRCodeDialog.setObjectName("ScanQRCodeDialog")
        ScanQRCodeDialog.resize(679, 93)
        self.verticalLayout = QtWidgets.QVBoxLayout(ScanQRCodeDialog)
        self.verticalLayout.setObjectName("verticalLayout")
        self.formLayout = QtWidgets.QFormLayout()
        self.formLayout.setObjectName("formLayout")
        self.addressLabel = QtWidgets.QLabel(ScanQRCodeDialog)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.addressLabel.setFont(font)
        self.addressLabel.setObjectName("addressLabel")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.addressLabel)
        self.addressValueLabel = QtWidgets.QLabel(ScanQRCodeDialog)
        self.addressValueLabel.setText("")
        self.addressValueLabel.setTextInteractionFlags(QtCore.Qt.LinksAccessibleByMouse|QtCore.Qt.TextSelectableByKeyboard|QtCore.Qt.TextSelectableByMouse)
        self.addressValueLabel.setObjectName("addressValueLabel")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.addressValueLabel)
        self.verticalLayout.addLayout(self.formLayout)
        self.errorLabel = QtWidgets.QLabel(ScanQRCodeDialog)
        self.errorLabel.setStyleSheet("color: rgb(255, 0, 0)")
        self.errorLabel.setObjectName("errorLabel")
        self.verticalLayout.addWidget(self.errorLabel, 0, QtCore.Qt.AlignHCenter)
        self.buttonBox = QtWidgets.QDialogButtonBox(ScanQRCodeDialog)
        self.buttonBox.setEnabled(True)
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.verticalLayout.addWidget(self.buttonBox, 0, QtCore.Qt.AlignHCenter)

        self.retranslateUi(ScanQRCodeDialog)
        self.buttonBox.accepted.connect(ScanQRCodeDialog.accept) # type: ignore
        self.buttonBox.rejected.connect(ScanQRCodeDialog.reject) # type: ignore
        QtCore.QMetaObject.connectSlotsByName(ScanQRCodeDialog)

    def retranslateUi(self, ScanQRCodeDialog):
        #  _translate = QtCore.QCoreApplication.translate
        ScanQRCodeDialog.setWindowTitle(_("Add address from QRCode"))
        self.addressLabel.setText(_("Address"))
import tikka.slots.pyqt.resources.icons.index_rc



# Copyright 2021 Vincent Texier <vit@free.fr>
#
# This software is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
from pathlib import Path

import yaml

from tikka.domains.entities.constants import CURRENCIES_FILENAME
from tikka.domains.entities.currency import Currency
from tikka.interfaces.adapters.repository.currencies import (
    CurrenciesRepositoryInterface,
)

ASSETS_PATH = Path(__file__).parent.joinpath("assets")
CURRENCIES_PATH = ASSETS_PATH.joinpath(CURRENCIES_FILENAME)


class FileCurrenciesRepository(CurrenciesRepositoryInterface):
    """
    FileCurrenciesRepository class
    """

    PARAMETER_NAME = "name"
    PARAMETER_SS58_FORMAT = "ss58_format"
    PARAMETER_ENTRY_POINTS = "entry_points"

    filepath = None

    def __init__(self):
        """
        Load Yaml file of currencies

        :return:
        """
        with open(
            Path(CURRENCIES_PATH).expanduser(), "r", encoding="utf-8"
        ) as filehandler:
            self.currencies = yaml.safe_load(filehandler)

    def get(self, code_name: str) -> Currency:
        __doc__ = (  # pylint: disable=redefined-builtin, unused-variable
            FileCurrenciesRepository.get.__doc__
        )
        parameters = self._parameters(code_name)
        return Currency(
            code_name,
            parameters[self.PARAMETER_NAME],
            parameters[self.PARAMETER_SS58_FORMAT],
        )

    def code_names(self) -> list:
        __doc__ = (  # pylint: disable=redefined-builtin, unused-variable
            FileCurrenciesRepository.code_names.__doc__
        )
        return list(self.currencies.keys())

    def names(self) -> list:
        __doc__ = (  # pylint: disable=redefined-builtin, unused-variable
            FileCurrenciesRepository.names.__doc__
        )
        names = []
        for code_name in self.currencies:
            names.append(self.currencies[code_name]["name"])

        return names

    def get_entry_point_urls(self, code_name: str) -> dict:
        """
        Return currency entry point urls

        :param code_name: Currency code name
        :return:
        """
        parameters = self._parameters(code_name)
        return parameters[self.PARAMETER_ENTRY_POINTS]

    def _parameters(self, code_name: str) -> dict:
        """
        Return currency parameters for code name

        :param code_name: Currency code name
        :return:
        """
        return self.currencies[code_name]



# Copyright 2021 Vincent Texier <vit@free.fr>
#
# This software is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
import logging
import sys
from typing import List, Optional

from PyQt5.QtCore import QTimer
from PyQt5.QtWidgets import QApplication, QListWidgetItem, QMainWindow, QWidget

from tikka.domains.application import Application
from tikka.domains.entities.constants import DATA_PATH
from tikka.domains.entities.events import ConnectionsEvent, CurrencyEvent
from tikka.domains.entities.node import Node
from tikka.domains.entities.technical_committee import TechnicalCommitteeProposal
from tikka.slots.pyqt.entities.constants import (
    TECHNICAL_COMMITTEE_SELECTED_ACCOUNT_ADDRESS,
)
from tikka.slots.pyqt.resources.gui.widgets.technical_committee_rc import (
    Ui_technicalCommitteeWidget,
)
from tikka.slots.pyqt.widgets.technical_committee_proposal import (
    TechnicalCommitteeProposalWidget,
)


class TechnicalCommitteeWidget(QWidget, Ui_technicalCommitteeWidget):
    """
    TechnicalCommitteeWidget class
    """

    def __init__(
        self, application: Application, parent: Optional[QWidget] = None
    ) -> None:
        """
        Init TechnicalCommitteeWidget instance

        :param application: Application instance
        :param parent: MainWindow instance
        """
        super().__init__(parent=parent)
        self.setupUi(self)

        self.application = application
        self._ = self.application.translator.gettext
        self.members: List[str] = []
        self.proposals: List[TechnicalCommitteeProposal] = []

        ##############################
        # ASYNC METHODS
        ##############################
        self.fetch_data_from_network_timer = QTimer()
        self.fetch_data_from_network_timer.timeout.connect(
            self.fetch_data_from_network_timer_function
        )

        # events
        self.accountComboBox.activated.connect(self.on_account_combobox_index_changed)
        self.refreshButton.clicked.connect(self._on_refresh_button_clicked)

        # subscribe to application events
        self.application.event_dispatcher.add_event_listener(
            CurrencyEvent.EVENT_TYPE_CHANGED, self._on_currency_event
        )
        self.application.event_dispatcher.add_event_listener(
            ConnectionsEvent.EVENT_TYPE_CONNECTED, self.on_connected_event
        )
        self.application.event_dispatcher.add_event_listener(
            ConnectionsEvent.EVENT_TYPE_DISCONNECTED, self.on_disconnected_event
        )

        # populate form
        self.fetch_data_from_network_timer.start()

    def init_member_list(self) -> None:
        """
        Init list of members

        :return:
        """
        self.membersListWidget.clear()

        for address in self.members:
            account = self.application.accounts.get_by_address(address)
            self.membersListWidget.addItem(
                f"{address}{' - ' + account.name if account is not None and account.name is not None else ''}"
            )

    def init_proposal_list(self) -> None:
        """
        Init list of proposals

        :return:
        """
        self.proposalsListWidget.clear()

        for proposal in self.proposals:
            item = QListWidgetItem(self.proposalsListWidget)
            self.proposalsListWidget.addItem(item)
            widget = TechnicalCommitteeProposalWidget(
                self.application, proposal, parent=self
            )
            item.setSizeHint(widget.minimumSizeHint())

            # Associate the custom widget to the list entry
            self.proposalsListWidget.setItemWidget(item, widget)

    def init_account_combo_box(self) -> None:
        """
        Init combobox with technical committee member accounts (with wallets)

        :return:
        """
        self.accountComboBox.clear()

        accounts = self.application.accounts.get_list()
        for account in accounts:
            if account.address in self.members and self.application.wallets.exists(
                account.address
            ):
                self.accountComboBox.addItem(
                    account.name if account.name is not None else account.address,
                    userData=account.address,
                )

        preference_account_address_selected = self.application.preferences.get(
            TECHNICAL_COMMITTEE_SELECTED_ACCOUNT_ADDRESS
        )
        if preference_account_address_selected is not None:
            preference_account_selected = self.application.accounts.get_by_address(
                preference_account_address_selected
            )
            if preference_account_selected is not None:
                index = self.accountComboBox.findData(
                    preference_account_address_selected
                )
                if index > -1:
                    self.accountComboBox.setCurrentIndex(index)
                else:
                    self.application.preferences.set(
                        TECHNICAL_COMMITTEE_SELECTED_ACCOUNT_ADDRESS, None
                    )

    def _on_refresh_button_clicked(self, _):
        """
        Refresh data from network when refresh button is clicked

        :param _:
        :return:
        """
        self.errorLabel.setText("")
        self.fetch_data_from_network_timer.start()

    def on_account_combobox_index_changed(self):
        """
        Triggered when account selection is changed

        :return:
        """
        address = self.accountComboBox.currentData()

        self.application.preferences_repository.set(
            TECHNICAL_COMMITTEE_SELECTED_ACCOUNT_ADDRESS,
            address,
        )

    def fetch_data_from_network_timer_function(self):
        """
        Fetch data of technical committee from network

        :return:
        """
        self.members = []
        try:
            self.members = self.application.technical_committee.network_members()
        except Exception as exception:
            self.errorLabel.setText(self._(str(exception)))

        try:
            self.proposals = self.application.technical_committee.network_proposals()
        except Exception as exception:
            self.errorLabel.setText(self._(str(exception)))

        self.init_member_list()
        self.init_proposal_list()
        self.init_account_combo_box()
        self.fetch_data_from_network_timer.stop()

    def _on_currency_event(self, _):
        """
        When a currency event is triggered

        :param _: CurrencyEvent instance
        :return:
        """
        self.fetch_data_from_network_timer.start()

    def on_connected_event(self, _):
        """
        Triggered when the network connection is connected

        :param _: ConnectionsEvent instance
        :return:
        """
        self.fetch_data_from_network_timer.start()

    def on_disconnected_event(self, _):
        """
        Triggered when the network connection is disconnected

        :param _: ConnectionsEvent instance
        :return:
        """
        self.fetch_data_from_network_timer.stop()
        self.init_member_list()
        self.init_proposal_list()


if __name__ == "__main__":
    logging.basicConfig(level=logging.DEBUG)
    qapp = QApplication(sys.argv)

    application_ = Application(DATA_PATH)

    main_window = QMainWindow()
    main_window.show()
    validator_node = Node(url="ws://localhost:9944")
    if application_.nodes.get(validator_node.url) is None:
        application_.nodes.add(validator_node)
    application_.nodes.set_current_url(validator_node.url)
    main_window.setCentralWidget(TechnicalCommitteeWidget(application_, main_window))

    sys.exit(qapp.exec_())



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'tikka/slots/pyqt/resources/gui/windows/configuration.ui'
#
# Created by: PyQt5 UI code generator 5.15.9
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_ConfigurationDialog(object):
    def setupUi(self, ConfigurationDialog):
        ConfigurationDialog.setObjectName("ConfigurationDialog")
        ConfigurationDialog.resize(418, 346)
        ConfigurationDialog.setLocale(QtCore.QLocale(QtCore.QLocale.English, QtCore.QLocale.UnitedStates))
        self.verticalLayoutWidget = QtWidgets.QWidget(ConfigurationDialog)
        self.verticalLayoutWidget.setGeometry(QtCore.QRect(9, 10, 401, 331))
        self.verticalLayoutWidget.setObjectName("verticalLayoutWidget")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.verticalLayoutWidget)
        self.verticalLayout.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout.setObjectName("verticalLayout")
        self.imageLabel = QtWidgets.QLabel(self.verticalLayoutWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.imageLabel.sizePolicy().hasHeightForWidth())
        self.imageLabel.setSizePolicy(sizePolicy)
        self.imageLabel.setMaximumSize(QtCore.QSize(100, 100))
        self.imageLabel.setSizeIncrement(QtCore.QSize(0, 0))
        self.imageLabel.setBaseSize(QtCore.QSize(0, 0))
        self.imageLabel.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.imageLabel.setText("")
        self.imageLabel.setPixmap(QtGui.QPixmap(":/images/config"))
        self.imageLabel.setScaledContents(True)
        self.imageLabel.setObjectName("imageLabel")
        self.verticalLayout.addWidget(self.imageLabel, 0, QtCore.Qt.AlignHCenter)
        self.groupBoxLanguage = QtWidgets.QGroupBox(self.verticalLayoutWidget)
        self.groupBoxLanguage.setStyleSheet("")
        self.groupBoxLanguage.setLocale(QtCore.QLocale(QtCore.QLocale.English, QtCore.QLocale.UnitedStates))
        self.groupBoxLanguage.setFlat(False)
        self.groupBoxLanguage.setObjectName("groupBoxLanguage")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.groupBoxLanguage)
        self.verticalLayout_4.setContentsMargins(-1, -1, -1, 0)
        self.verticalLayout_4.setSpacing(0)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.languageRadioButtonsLayout = QtWidgets.QVBoxLayout()
        self.languageRadioButtonsLayout.setSpacing(0)
        self.languageRadioButtonsLayout.setObjectName("languageRadioButtonsLayout")
        self.languageHelpLabel = QtWidgets.QLabel(self.groupBoxLanguage)
        font = QtGui.QFont()
        font.setItalic(True)
        self.languageHelpLabel.setFont(font)
        self.languageHelpLabel.setObjectName("languageHelpLabel")
        self.languageRadioButtonsLayout.addWidget(self.languageHelpLabel)
        self.verticalLayout_4.addLayout(self.languageRadioButtonsLayout)
        self.verticalLayout.addWidget(self.groupBoxLanguage)
        self.groupBoxCurrency = QtWidgets.QGroupBox(self.verticalLayoutWidget)
        self.groupBoxCurrency.setStyleSheet("")
        self.groupBoxCurrency.setLocale(QtCore.QLocale(QtCore.QLocale.English, QtCore.QLocale.UnitedStates))
        self.groupBoxCurrency.setFlat(False)
        self.groupBoxCurrency.setObjectName("groupBoxCurrency")
        self.verticalLayout_5 = QtWidgets.QVBoxLayout(self.groupBoxCurrency)
        self.verticalLayout_5.setContentsMargins(-1, 0, -1, 0)
        self.verticalLayout_5.setSpacing(0)
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.currencyRadioButtonsLayout = QtWidgets.QVBoxLayout()
        self.currencyRadioButtonsLayout.setSpacing(0)
        self.currencyRadioButtonsLayout.setObjectName("currencyRadioButtonsLayout")
        self.verticalLayout_5.addLayout(self.currencyRadioButtonsLayout)
        self.verticalLayout.addWidget(self.groupBoxCurrency)
        self.randomConnectioncheckBox = QtWidgets.QCheckBox(self.verticalLayoutWidget)
        font = QtGui.QFont()
        font.setBold(False)
        font.setWeight(50)
        self.randomConnectioncheckBox.setFont(font)
        self.randomConnectioncheckBox.setChecked(True)
        self.randomConnectioncheckBox.setObjectName("randomConnectioncheckBox")
        self.verticalLayout.addWidget(self.randomConnectioncheckBox)
        self.buttonBox = QtWidgets.QDialogButtonBox(self.verticalLayoutWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.buttonBox.sizePolicy().hasHeightForWidth())
        self.buttonBox.setSizePolicy(sizePolicy)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Save)
        self.buttonBox.setCenterButtons(True)
        self.buttonBox.setObjectName("buttonBox")
        self.verticalLayout.addWidget(self.buttonBox)

        self.retranslateUi(ConfigurationDialog)
        QtCore.QMetaObject.connectSlotsByName(ConfigurationDialog)

    def retranslateUi(self, ConfigurationDialog):
        #  _translate = QtCore.QCoreApplication.translate
        ConfigurationDialog.setWindowTitle(_("Configuration"))
        self.groupBoxLanguage.setTitle(_("Language"))
        self.languageHelpLabel.setText(_("Restart application to apply language change"))
        self.groupBoxCurrency.setTitle(_("Currency"))
        self.randomConnectioncheckBox.setText(_("Connect randomly to a node at start"))
import tikka.slots.pyqt.resources.icons.index_rc



# Copyright 2021 Vincent Texier <vit@free.fr>
#
# This software is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
from typing import List, Optional

from substrateinterface import Keypair

from tikka.domains.entities.authorities import Authority, AuthorityStatus
from tikka.domains.entities.node import Node
from tikka.domains.nodes import Nodes
from tikka.domains.smiths import Smiths
from tikka.interfaces.adapters.network.authorities import NetworkAuthoritiesInterface
from tikka.interfaces.adapters.repository.authorities import (
    AuthoritiesRepositoryInterface,
)


class Authorities:

    """
    Authorities domain class
    """

    def __init__(
        self,
        repository: AuthoritiesRepositoryInterface,
        network: NetworkAuthoritiesInterface,
        nodes: Nodes,
        smiths: Smiths,
    ):
        """
        Init Authorities domain

        :param repository: AuthoritiesRepositoryInterface instance
        :param network: NetworkAuthoritiesInterface instance
        :param nodes: Nodes domain instance
        :param smiths: Smiths domain instance
        """
        self.network = network
        self.repository = repository
        self.nodes = nodes
        self.smiths = smiths

    @staticmethod
    def create(identity_index: int, status: AuthorityStatus):
        """
        Return Authority instance from parameters

        :param identity_index: Identity index
        :param status: AuthorityStatus enum
        :return:
        """
        return Authority(identity_index, status)

    def add(self, authority: Authority) -> None:
        """
        Add Authority in repository

        :param authority: Authority instance
        :return:
        """
        self.repository.add(authority)

    def get(self, identity_index: int) -> Optional[Authority]:
        """
        Get Authority by identity index from repository

        :param identity_index: Identity index
        :return:
        """
        return self.repository.get(identity_index)

    def update(self, authority: Authority):
        """
        Update authority in repository

        :param authority: Authority instance
        :return:
        """
        self.repository.update(authority)

    def delete(self, identity_index: int) -> None:
        """
        Delete Authority in repository

        :param identity_index: Identity index to delete
        :return:
        """
        self.repository.delete(identity_index)

    def exists(self, identity_index: int) -> bool:
        """
        Return True if Authority exists in repository

        :param identity_index: Identity index to check
        :return:
        """
        return self.repository.exists(identity_index)

    def list(self) -> List[Authority]:
        """
        Return list of all Authority in repository

        :return:
        """
        return self.repository.list()

    def get_status(self, identity_index: int) -> AuthorityStatus:
        """
        Return AuthorityStatus enum for identity_index

        :param identity_index: Identity index
        :return:
        """
        if not self.repository.exists(identity_index):
            return AuthorityStatus.OFFLINE

        authority = self.repository.get(identity_index)
        if authority is None:
            return AuthorityStatus.OFFLINE

        return authority.status

    def network_rotate_keys(self, node: Node) -> Optional[str]:
        """
        Change node session keys and return them

        :param node: Node instance
        :return:
        """
        session_keys = self.network.rotate_keys()
        if session_keys is not None:
            node.session_keys = session_keys
            self.nodes.update(node)
        return session_keys

    def network_has_session_keys(self, session_keys: str) -> Optional[bool]:
        """
        Return True if the current node keystore store private session keys corresponding to public session_keys

        :param session_keys: Session public keys (hex string "0x123XYZ")
        :return:
        """
        return self.network.has_session_keys(session_keys)

    def network_publish_session_keys(self, keypair: Keypair, session_keys: str) -> None:
        """
        Set/Change in blockchain the session public keys for the Keypair account

        :param keypair: Owner Keypair
        :param session_keys: Session public keys (hex string "0x123XYZ")
        :return:
        """
        return self.network.publish_session_keys(keypair, session_keys)

    def network_go_online(self, keypair: Keypair) -> None:
        """
        Start writing blocks with smith account from keypair

        :param keypair: Smith account Keypair
        :return:
        """
        return self.network.go_online(keypair)

    def network_go_offline(self, keypair: Keypair) -> None:
        """
        Stop writing blocks with smith account from keypair

        :param keypair: Smith account Keypair
        :return:
        """
        return self.network.go_offline(keypair)

    def network_get_status(self, identity_index: int) -> AuthorityStatus:
        """
        Return AuthorityStatus enum for identity_index

        :param identity_index: Identity index
        :return:
        """
        return self.network.get_status(identity_index)

    def network_get_all(self) -> None:
        """
        Get all authorities by status

        :return:
        """
        network_authorities = self.network.get_all()

        # list of known smith
        smith_identity_indice = [smith.identity_index for smith in self.smiths.list()]
        authorities = self.list()

        # purge obsolete authorities
        for authority in authorities:
            if authority.identity_index not in smith_identity_indice:
                self.delete(authority.identity_index)

        for authority in authorities:
            if (
                authority.identity_index
                in network_authorities[AuthorityStatus.ONLINE.value]
            ):
                authority.status = AuthorityStatus.ONLINE
            elif (
                authority.identity_index
                in network_authorities[AuthorityStatus.INCOMING.value]
            ):
                authority.status = AuthorityStatus.INCOMING
            elif (
                authority.identity_index
                in network_authorities[AuthorityStatus.OUTGOING.value]
            ):
                authority.status = AuthorityStatus.OUTGOING
            else:
                authority.status = AuthorityStatus.OFFLINE
            self.update(authority)

        # new smiths
        authority_identity_indice = [
            authority.identity_index for authority in authorities
        ]
        for identity_index in smith_identity_indice:
            if identity_index not in authority_identity_indice:
                authority = self.create(identity_index, AuthorityStatus.OFFLINE)
                if (
                    authority.identity_index
                    in network_authorities[AuthorityStatus.ONLINE.value]
                ):
                    authority.status = AuthorityStatus.ONLINE
                elif (
                    authority.identity_index
                    in network_authorities[AuthorityStatus.INCOMING.value]
                ):
                    authority.status = AuthorityStatus.INCOMING
                elif (
                    authority.identity_index
                    in network_authorities[AuthorityStatus.OUTGOING.value]
                ):
                    authority.status = AuthorityStatus.OUTGOING
                self.add(authority)
